/*
** bgikernl.h
**
** Mappings from the source graphics calls to the Borland BGI
** graphics kernel.
**
** Works ONLY with Borland C/C++ and BGI
**
*/

#ifndef BGI_KERNEL
#define BGI_KERNEL

#include    <graphics.h>                /* Borland graphics stuff           */

#ifndef Byte
#define Byte    unsigned char           /* 8-bit quantity                   */
#endif

#ifndef Word
#define Word    unsigned short          /* 'short'-size (16 or 32 bits)     */
#endif

#define alignLeft       0               /* "TextAlign" - Horizontal         */
#define alignCenter     1
#define alignRight      2

#define alignBaseline   0               /* "TextAlign" - Vertical           */
#define alignBottom     0
#define alignMiddle     2
#define alignTop        3

#define KEYBOARDINTR    9               /* DOS Keyboard interrupt           */

#define zXORz           1               /* Rasterop modes                   */
#define zREPz           0
#define zORz            2
#define zANDz           3
#define zNREPz          4
#define xREPx           0               /* In BGI, trasparent ops are same  */

#define ruleWinding     1               /* polygon fill rules (ignored)     */
#define ruleOddEven     0


typedef struct {
    int X, Y;
} point;


typedef struct {
    int Xmin, Ymin, Xmax, Ymax;
} rect;

#define cNormal         0  /* "TextFace" definitions         */
#define cBold           1
#define cItalic         2
#define cUnderline      4
#define cStrikeout      8
#define cHalftone      16
#define cMirrorX       32
#define cMirrorY       64
#define cProportional 128

typedef struct _grafMap {     /* "grafMap" Data Structure      */
    short        pixBytes;      /* Bytes per scan line           */
    Word         pixWidth;      /* Pixels horizontal             */
    Word         pixHeight;     /* Pixels vertical               */
    short        pixResX;       /* Pixels per inch horzontally   */
    short        pixResY;       /* Pixels per inch vertically    */
    short        pixBits;       /* Color bits per pixel          */
    short        pixPlanes;     /* Color planes per pixel        */
} grafMap;

typedef struct {
    rect    portClip;
    short   txFace;
    grafMap *portMap;
} grafPort;

/*
** Define the event record
*/
typedef struct _mwEvent {   /* Event record structure         */
    short    eventTime;      /* System time stamp            */
    Byte     eventType;      /* Event type flags             */
    Byte     eventSource;    /* Event source device ID       */
    short    eventChar;      /* Keyboard Character code      */
    short    eventScan;      /* Keyboard Scan code           */
    short    eventState;     /* Keyboard State info          */
    short    eventButtons;   /* Positional device buttons    */
    short    eventX;         /* Positional device X          */
    short    eventY;         /* Positional device Y          */
    short    eventUser[2];   /* User defined event data      */
} mwEvent;
                            /* Event Type and Mask flags      */
#define evntNONE    0x00     /* no events pending            */
#define evntMOVE    0x01     /* input movement               */
#define evntPRESS   0x02     /* button press                 */
#define evntRELEASE 0x04     /* button release               */
#define evntKEYDN   0x08     /* key press                    */
#define evntKEYUP   0x10     /* key release                  */
#define evntUSER1   0x20     /* user program event 1         */
#define evntUSER2   0x40     /* user program event 2         */
#define evntALL     0xFF     /* MaskEvent() - enable all     */

                           /* "eventButtons" mouse buttons   */
#define swLeft         1
#define swRight        2
#define swMiddle       4     /* (not on 2 button mice)       */
#define swAll          7

#define TextPg0         0
#define GrafPg0       256
#define GrafPg1       257

typedef unsigned char image;    /* "image" data type equate  */

typedef struct _imageHeader { /* Image Header record structure  */
    short    imWidth;        /* Pixel width (X)              */
    short    imHeight;       /* Pixel height (Y)             */
    char     imAlign;        /* Image alignment              */
    char     imFlags;        /* Image flags                  */
    short    imRowBytes;     /* Bytes per row                */
    short    imBits;         /* Bits per pixel               */
    short    imPlanes;       /* Planes per pixel             */
    /*  image    imData[]; */    /* (image data, variable length)*/
} imageHeader;

typedef struct _polyHead {  /* Polygon "header" structure       */
    Word     polyBgn;       /* beginning index                  */
    Word     polyEnd;       /* ending index                     */
    rect     polyRect;      /* boundry limits                   */
} polyHead;

typedef struct _fontrcd {
    char    fontFacing;     /* Synthesized font facing flags        */
    long    fontSize;       /* Buffer size needed to hold font      */
    short   fontFlags;      /* Font flags                           */
    short   chWidth;        /* Fixed space character width          */
    short   chHeight;       /* Character height                     */
    short   ascent;         /* Ascent                               */
    short   descent;        /* Descent                              */
    short   fontnum;        /* BGI font number                      */
} fontRcd;

/*
** Function prototypes
** Some functions we can map directly via macros
*/

void    BackColor(int color);
int     CharWidth(char c);
void    ClipRect(rect *);
void    DrawChar(char ch);
void    DrawString(char *s);
void    DrawText(char *str, int start, int count);
void    DupRect(rect *r1, rect *r2);
int     EqualRect(rect *r1, rect *r2);
void    EraseArc(rect *r, int startang, int endang);
void    EraseRect(rect *r);
void    EraseOval(rect *r);
void    ErasePoly(int count, polyHead *hdr, point *points);
void    EventQueue(int state);
int     FileLoad(char *fname, void *buf, Word size);
void    FillRule(int rule);
void    FrameArc(rect *r, int bangle, int eangle);
void    FrameOval(rect *r);
void    FramePoly(int count, polyHead *hdr, point *points);
void    FrameRect(rect *r);
int     GetPixel(int, int);
void    GetPort(grafPort **port);
void    HideCursor(void);
long    ImageSize(rect *r);
int     InceptRect(rect *r1, rect *r2, rect *dest);
void    InitKeyboardEventHandler(void);
int     InitMouse(int driver);
void    InitMouseEventHandler(void);
void    InitPorts(rect *r);
void    InsetRect(rect *r, int dx, int dy);
void    InvertRect(rect *r);
int     KeyEvent(int wait, mwEvent *evnt);
void    LimitMouse(int xmin, int ymin, int xmax, int ymax);
void    LineTo(int x, int y);
void    MaskEvent(int mask);
void    MoveCursor(int x, int y);
void    MoveTo(int x, int y);
void    OffsetPoly(int count, polyHead *hdr, point *points, int x, int y);
void    OffsetRect(rect *r, int dx, int dy);
void    PaintOval(rect *r);
void    PaintPoly(int count, polyHead *hdr, point *points);
void    PaintRect(rect *r);
void    PenColor(int color);
void    PenDash(int dash);
void    PenMode(int mode);
void    PenSize(int diax, int diay);
void    ProtectRect(rect *r);
int     PtInPoly(point *pt,
                 int polycnt,
                 polyHead *hdr,
                 point *polypt,
                 int pixw,
                 int pixh);
int     PtInRect(point *pt, rect *r, int pixw, int pixh);
int     QueryColors(void);
void    QueryCursor(int *, int *, int *, int *);
void    RasterOp(int rop);
void    ReadImage(rect *r, image *dst);
void    ScreenRect(rect *);
void    ScrollRect(rect *r, int dx, int dy);
void    SetDisplay(int page);
void    SetPixel(int, int);
void    SetPt(point *pt, int x, int y);
void    SetRect(rect *r, int x1, int y1, int x2, int y2);
int     ShiftRect(rect *r, int dx, int dy, rect *r1, rect *r2);
void    ShowCursor(void);
void    StopEvent(void);
void    StopGraphics(void);
void    StopMouse(void);
int     StringWidth(char *str);
void    TextAlign(int xalign, int yalign);
void    TextFace(int face);
void    TextMode(int rop);
void    TextSize(int width, int height);
int     TextWidth(char *str, int start, int count);
void    TrackCursor(int flag);
void    WriteImage(rect *r, image *src);

#endif
