/*
** Iconedit.c
**
** Utility for building simple TWS icons
**
** Copyright 1994, 1995 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include <float.h>
#include <math.h>
#include <dir.h>
#include <string.h>
#include "smwindow.h"
#include "smtypes.h"
#include "smevent.h"
#include "smgraph.h"
#include "smmenu.h"
#include "smlabel.h"
#include "smbutton.h"
#include "smtext.h"
#include "smbutton.h"
#include "smcolor.h"
#include "filedial.h"
#include "smyesno.h"
#include "smync.h"
#include "smok.h"
#include "smwutil.h"
#include "smfont.h"
#include "smpixmap.h"
#include <smborder.h>
#include <smeditst.h>

/*
** Borlandc requires extra stack
*/
#ifdef __TURBOC__
extern unsigned _stklen = 16384U;
#endif

#include    "iconedit.h"

int fatbitsize;                         /* Width of a fat bit in pixels     */
int cindex, bindex;

LabelType   *forelabel, *backlabel;     /* Labels for foreground/background */
PixmapType  *undopm;
PixmapType  *SelectPixmap;

int     MAXSIZEX, MAXSIZEY;

void main(int argc, char *argv[])
{
    WindowType  *w;                     /* The icon editor window           */
    RectType    r;                      /* Rectangle for the window         */
    EventType   ev;                    /* Event variable                   */
    int         msg;                    /* Event return value               */
    IconEditData    IconData;           /* "Icon" data (pixmap, actually)   */
    int         colorbuttonsize;        /* Width of the color select buttons*/
    MenuType    *winmenu;
    MenuType    *filemenu;
    MenuType    *editmenu;

    /*
    ** Initialize and open the application desktop
    */
    SM_Init(0,0);

    /*
    ** Set up the window menus
    */
    winmenu = SM_CreateMenu();
    filemenu = SM_CreateMenu();
    editmenu = SM_CreateMenu();
    SM_AddMenuItem(winmenu, SUBMENU_ITEM, "File", NULL, filemenu);
    SM_AddMenuItem(winmenu, SUBMENU_ITEM, "Edit", NULL, editmenu);
    SM_AddMenuItem(winmenu, ACTION_ITEM, "Help", DisplayHelp, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "Open", ReadPixmap, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "New", NewPixmap, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "Save", SavePixmap, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "SaveAs", WritePixmap, NULL);
    SM_AddMenuItem(filemenu, MENU_DIVIDER, NULL, NULL, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "About", About, NULL);
    SM_AddMenuItem(filemenu, MENU_DIVIDER, NULL, NULL, NULL);
    SM_AddMenuItem(filemenu, ACTION_ITEM, "Quit", IconExit, NULL);
    SM_AddMenuItem(editmenu, ACTION_ITEM, "Clear", ClearPixmap, NULL);
    SM_AddMenuItem(editmenu, ACTION_ITEM, "Undo", UndoPixmap, NULL);

    SM_OpenApplication("Icon Edit", winmenu);

    /*
    ** Make each square in the drawing area 5 pixels wide
    */
    fatbitsize = 5;

    /*
    ** Find the largest pixmap we can draw based on screen size
    */
    MAXSIZEX = MAXSIZEY = (SM_GetDisplayWidth() < 800) ? 64 : 80;

    /*
    ** The workspace content region contains the drawing area, plus
    ** room for the color buttons and control buttons to the right of the
    ** canvas.  Default to 64 pixels square.
    */
    IconData.isizex = MAXSIZEX;
    IconData.isizey = MAXSIZEY;

    /*
    ** Make the color buttons just fit along the bottom of the fatbit pixmap
    */
    colorbuttonsize = (IconData.isizex * fatbitsize - 15)/16;

    /*
    ** Get the workspace window
    */
    w = SM_WorkspaceWindow();

    /*
    ** Create the undo pixmap
    */
    r.Xmin = r.Ymin = 0;
    r.Xmax = r.Ymax = 63;
    undopm = SM_CreatePixmap(NULL, &r, 0);

    /*
    ** Create a graphics state so we can draw in the window.  Make the canvas
    ** just big enough for the fatbit pixmap.
    */
    r.Xmin = 67;
    r.Ymin = 5;
    r.Ymax = r.Ymin + IconData.isizex*fatbitsize + 2*SM_GetDefaultBevelDepth();
    r.Xmax = r.Xmin + IconData.isizex*fatbitsize + 2*SM_GetDefaultBevelDepth();
    GR_CreateGraphState(w, &r, GRRECESSBORDER);

    /*
    ** Create the actual-size pixmap, initialize some program variables, colors, etc.
    */
    r.Xmin = r.Xmax + 5;
    r.Ymax = r.Ymin + IconData.isizey - 1;
    r.Xmax = r.Xmin + IconData.isizex - 1;
    IconData.pm = SM_CreatePixmap(w, &r, SCULPTED);
    IconData.modified = False;
    IconData.filename = NULL;
    IconData.tool = PENTOOL;
    bindex = 0;
    cindex = SMWHITE;
    IconData.backcolor = SM_GetSystemColor(bindex);
    IconData.drawcolor = SM_GetSystemColor(cindex);

    /*
    ** Create the draw tool buttons
    */
    r.Xmin = 3;
    r.Xmax = r.Xmin + 32;
    r.Ymin = 10;
    r.Ymax = r.Ymin + 32;
    SetToolButtons(w, &r);

    /*
    ** Build the color buttons...
    */
    CreateColorButtons(w, &IconData, colorbuttonsize);

    /*
    ** Set the graphics background color
    */
    GR_SetBackgroundColor(w, SM_GetSystemColor(bindex));
    GR_SetDrawColor(w, SM_GetSystemColor(cindex));

    /*
    ** Since we're working in the workspace window, which is 'opened' as
    ** soon as it is initialized, the drawing procedures and data aren't
    ** attached when the window is first drawn.  Attach a draw procedure
    ** and data here, then force the workspace to redraw
    */
    SM_SetWindowProc(w, IconDraw);
    SM_SetDrawProc(w, DrawEditWin);
    SM_SetWindowData(w, &IconData);
    SM_RedrawWindow(w);
    SM_SetPrintscreenProc(NULL, NULL, "panel.pcx", PrntScrn);

    /*
    ** The ubiquitous event loop...
    */
    while (True) {
        SM_GetNextEvent(&ev);
        msg = SM_ProcessEvent(&ev);
        if (msg == KEYPRESS) {
            if (ev.ASCII == 'q') SM_Exit(NULL);
        }
    }
}



/***************************************************************************
**
** NewPixmap
**
** Start a brand-new pixmap
*/
int NewPixmap(WindowType *w)
{
    IconEditData    *icondata;
    char            msg[80];

    /*
    ** See if the existing drawing has unsaved changes...
    */
    icondata = (IconEditData *)SM_GetUserData(w);
    if (icondata->modified) {
        sprintf(msg,"Discard changes to %s",icondata->filename);
        if (SM_YesNoDialog(msg, "Yes", "Cancel")) {
            return 1;
        }
    }
    SetPixmapSize(icondata, SetAndClearPixmap);
    return 1;
}


static int  newwidth, newheight;
WindowType  *sizewin;


/***************************************************************************
**
** SetPixmapSize
**
** Displays a dialog prompting for the rows, columns size of the pixmap.
** The function 'f' is called if the user chooses the OK button.  Probably
** this is the function that sets the pixmap size.
*/
void SetPixmapSize(IconEditData *icondata, int (*f)())
{
    RectType    r;

    /*
    ** Create the dialog window
    */
    SM_SetRectangle(&r, 250, 175, 250, 150);
    sizewin = SM_NewWindow(&r, "Set Pixmap Size", DIALOG, NULL, NULL);

    /*
    ** Put in labels and edit strings to enter data
    */
    SM_SetRectangle(&r, 1, 1, SM_GetContentWidth(sizewin)-2, 25);
    SM_CreateLabel(sizewin,
                   &r,
                   "Enter width, height of pixmap (pixels)",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   0,
                   NULL);
    SM_SetRectangle(&r, -250, 40, -500, 25);
    SM_CreateLabel(sizewin,
                   &r,
                   "Width:",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   0,
                   NULL);
    SM_SetRectangle(&r, -525, 42, -800, 20);
    SM_CreateEditstring(sizewin,
                        &r,
                        "",
                        4,
                        (void *)&newwidth,
                        IntFilter);     /* Forces all characters to digits  */
    SM_SetRectangle(&r, -250, 65, -500, 20);
    SM_CreateLabel(sizewin,
                   &r,
                   "Height:",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   0,
                   NULL);
    SM_SetRectangle(&r, -525, 65, -800, 20);
    SM_CreateEditstring(sizewin,
                        &r,
                        "",
                        4,
                        (void *)&newheight,
                        IntFilter);     /* Forces all characters to digits  */
    SM_SetRectangle(&r, 5, 100, -5, 3);
    SM_CreateBorder(sizewin,
                    &r,
                    NULL,
                    0,
                    twsBORDERHLINE,
                    twsBORDERSCULPT,
                    1,
                    SM_GetSystemColor(SMDARKGRAY));
    SM_SetRectangle(&r, -200, 110, -495, 25);
    SM_CreateButton(sizewin,
                    &r,
                    "OK",
                    (void *)icondata,
                    f);
    SM_SetRectangle(&r, -505, 110, -800, 25);
    SM_CreateButton(sizewin,
                    &r,
                    "Cancel",
                    NULL,
                    PixmapSizeCancel);
    SM_OpenWindow(sizewin);
}



/***************************************************************************
**
** SetAndClearPixmap
**
** Changes the pixmap size based on user input, and clears the pixmap
*/
int SetAndClearPixmap(ButtonType *b)
{
    IconEditData    *icondata;
    RectType        r;
    WindowType      *parent;

    /*
    ** IMPORTANT!  We *must* get the data from the button *BEFORE* we close
    ** the window, because the button belongs to the window and it will be
    ** CLOBBERED when the window is closed!
    */
    icondata = (IconEditData *)SM_GetButtonData(b);

    /*
    ** NOW it's safe to close the window
    */
    SM_CloseWindow(sizewin);
    if (newheight <= 0
    || newheight > MAXSIZEY
    || newwidth <= 0
    || newwidth > MAXSIZEX) {
        SM_OKDialog("New pixmap height is out of bounds");
        return;
    }
    icondata->isizex = newwidth;
    icondata->isizey = newheight;
    /*
    ** Get the parent window for the new pixmap from the old pixmap
    */
    parent = SM_GetGadgetWindow(icondata->pm);

    /*
    ** Now clear the pixmaps
    */
    SM_DestroyPixmap(icondata->pm);
    SM_SetRectangle(&r,
                    72 + icondata->isizex*fatbitsize + 2*SM_GetDefaultBevelDepth(),
                    5,
                    newwidth,
                    newheight);
    icondata->pm = SM_CreatePixmap(parent, &r, SCULPTED);
    /*
    ** Clear the pixmap to the current background color
    */
    SM_ClearPixmap(icondata->pm, icondata->backcolor);
    SetWindowSize(parent);
    SM_RedrawWindow(parent);

    /*
    ** Have to return 1 (True) because this is the return value of a gadget
    ** callback, and we want to prevent the event handler from processing
    ** any more gadgets in this window
    */
    return 1;
}


                    
/****************************************************************************
**
** PixmapSizeCancel
**
** Just close the size window without changing anything
*/
int PixmapSizeCancel(ButtonType *b)
{
    SM_CloseWindow(sizewin);
}



/***************************************************************************
**
** IntFilter
**
** This is a callback for an editstring gadget.  This function blocks any
** characters that aren't digits [0..9].  Returning a non-zero value
** causes the editstring to ignore the event.
*/
int IntFilter(EditstringType *str, EventType *event)
{
    *(int *)(str->data) = atoi(SM_GetEditstringString(str));
    if (event->ASCII > 32 && (event->ASCII < '0' || event->ASCII > '9')) {
        return 1;
    }
    return 0;
}



/***************************************************************************
**
** SetToolButtons
**
** Initialize the tool selection buttons
*/
void SetToolButtons(WindowType *w, RectType *r)
{
    ButtonType  *LineButton,
                *BoxButton,
                *CircleButton,
                *PenButton,
                *FillButton;
    PixmapType  *PenPixmap;
    PixmapType  *LinePixmap;
    PixmapType  *BoxPixmap;
    PixmapType  *CirclePixmap;
    RectType    brect;

    /*
    ** brect is the boundary for the decorative panel to be drawn around
    ** all the tool buttons.
    */
    brect.Xmin = r->Xmin;
    brect.Ymin = r->Ymin;

    r->Xmin += 10;
    r->Ymin += 15;
    r->Xmax += 10;
    r->Ymax += 15;

    PenButton = SM_CreateButton(w,
                                r,
                                NULL,
                                (void *)PENTOOL,
                                SetTool);

    PenPixmap = SM_ReadPixmap("..\\icons\\pen.icn");
    SM_SetPixmapBlocking(PenPixmap, True);
    SM_SetButtonPixmap(PenButton, PenPixmap, 0, 0);

    r->Ymin = r->Ymax + 1;
    r->Ymax = r->Ymin + 32;
    LineButton = SM_CreateButton(w,
                                 r,
                                 NULL,
                                 (void *)LINETOOL,
                                 SetTool);
    LinePixmap = SM_ReadPixmap("..\\icons\\line.icn");
    SM_SetPixmapBlocking(LinePixmap, True);
    SM_SetButtonPixmap(LineButton, LinePixmap, 0, 0);
    r->Ymin = r->Ymax + 1;
    r->Ymax = r->Ymin + 32;
    BoxButton = SM_CreateButton(w,
                                r,
                                NULL,
                                (void *)BOXTOOL,
                                SetTool);
    BoxPixmap = SM_ReadPixmap("..\\icons\\box.icn");
    SM_SetPixmapBlocking(BoxPixmap, True);
    SM_SetButtonPixmap(BoxButton, BoxPixmap, 0, 0);
    r->Ymin = r->Ymax + 1;
    r->Ymax = r->Ymin + 32;
    CircleButton = SM_CreateButton(w,
                                   r,
                                   NULL,
                                   (void *)CIRCLETOOL,
                                   SetTool);
    CirclePixmap = SM_ReadPixmap("..\\icons\\circle.icn");
    SM_SetPixmapBlocking(CirclePixmap, True);
    SM_SetButtonPixmap(CircleButton, CirclePixmap, 0, 0);
    r->Ymin = r->Ymax + 1;
    r->Ymax = r->Ymin + 32;
    FillButton = SM_CreateButton(w,
                                 r,
                                 "Fill",
                                 (void *)FILLTOOL,
                                 SetTool);
    SM_SetButtonActive(FillButton, False);

    /*
    ** Create a border around all the tools
    */
    brect.Xmax = r->Xmax + 10;
    brect.Ymax = r->Ymax + 10;
    SM_CreateBorder(w,
                    &brect,
                    "Tools",
                    twsBORDERLEFT,
                    twsBORDERBOX,
                    twsBORDERLRIDGE,
                    1,
                    SM_GetSystemColor(SMDARKGRAY));

    /*
    ** Create a pixmap to show the currently selected tool
    */
    r->Ymin = brect.Ymax + 25;
    r->Ymax = r->Ymin + 31;
    SelectPixmap = SM_CreatePixmap(w, r, 0);
    SM_CopyPixmap(SelectPixmap, PenPixmap);
    brect.Ymin = r->Ymin - 15;
    brect.Ymax = r->Ymax + 10;
    SM_CreateBorder(w,
                    &brect,
                    NULL,
                    twsBORDERLEFT,
                    twsBORDERBOX,
                    twsBORDERLRIDGE,
                    1,
                    SM_GetSystemColor(SMDARKGRAY));
}



/******************************************************************************
**
** CreateColorButtons
**
** Initialize the color selection buttons at the bottom of the screen
*/
void CreateColorButtons(WindowType *w, IconEditData *IconData, int colorbuttonsize)
{
    RectType    r, labelr;
    int         i;

    /*
    ** Put a decorative line between screen & color buttons
    */
    r.Ymin = SM_GetContentDepth(w) - 2*colorbuttonsize - 12;
    r.Ymax = r.Ymin + 3;
    r.Xmin = 5;
    r.Xmax = SM_GetContentWidth(w) - 5;
    SM_CreateBorder(w,
                    &r,
                    NULL,
                    0,
                    twsBORDERHLINE,
                    twsBORDERSCULPT,
                    1,
                    SM_GetSystemColor(SMDARKGRAY));
    r.Xmin = 3*colorbuttonsize;
    r.Ymax = SM_GetContentDepth(w) - 5;
    r.Xmax = r.Xmin + SM_StringWidth("Foreground ", SM_GetSystemFont(), TXTPROPORTIONAL);
    r.Ymin = r.Ymax - colorbuttonsize;
    SM_CreateLabel(w,
                   &r,
                   "Background",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   0,
                   NULL);
    r.Ymax = r.Ymin - 1;
    r.Ymin = r.Ymax - colorbuttonsize;
    SM_CreateLabel(w,
                   &r,
                   "Foreground",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   0,
                   NULL);


    r.Xmin = r.Xmax + 5;
    r.Ymax = labelr.Ymax = SM_GetContentDepth(w) - 5;
    for (i = 0; i < 32; i++) {
        r.Xmax = r.Xmin + colorbuttonsize;
        r.Ymin = r.Ymax - colorbuttonsize;
        /*
        ** The following makes the buttons alternate into two layers
        */
        if (i % 2) {
            IconData->colorbuttons[i] = SM_CreateButton(w,
                                     &r,
                                     NULL,
                                     (void *)(i/2),
                                     SetDrawColor);
            r.Ymax = SM_GetContentDepth(w) - 5;
            r.Ymin = r.Ymax - colorbuttonsize;
            r.Xmin = r.Xmax + 1;
        }
        else {
            IconData->colorbuttons[i] = SM_CreateButton(w,
                                     &r,
                                     NULL,
                                     (void *)(i/2),
                                     SetBackColor);
            r.Ymax = r.Ymin - 1;
            r.Ymin = labelr.Ymin = r.Ymax - colorbuttonsize;
        }
        if (i > 29) {
            SM_SetButtonColor(IconData->colorbuttons[i], SM_GetSystemColor(SMWHITE));
            SM_SetButtonPushedColor(IconData->colorbuttons[i], SM_GetSystemColor(SMWHITE));
            SM_SetButtonData(IconData->colorbuttons[i], (void *)SMWHITE);
        }
        else {
            SM_SetButtonColor(IconData->colorbuttons[i], SM_GetSystemColor(i/2));
            SM_SetButtonPushedColor(IconData->colorbuttons[i], SM_GetSystemColor(i/2));
        }

    }
    /*
    ** Set up labels to display the selected foreground and background colors
    */
    labelr.Xmin = r.Xmax + 5;
    labelr.Xmax = labelr.Xmin + (labelr.Ymax - labelr.Ymin);
    /*
    ** The two labels occupy the same space, but the background label is
    ** bigger.  We're relying on the fact the TWS draws gadgets in the same
    ** order that they're created to ensure that the foreground drawing
    ** color label is always on top of the background color label.
    */
    backlabel = SM_CreateLabel(w,
                               &labelr,
                               NULL,
                               NULL,
                               SM_GetSystemFont(),
                               0,
                               False,
                               False,
                               SCULPTED,
                               &bindex);
    SM_SetGadgetBackcolor(backlabel, IconData->backcolor);
    GR_InsetRect(&labelr, 8, 8);
    forelabel = SM_CreateLabel(w,
                               &labelr,
                               NULL,
                               NULL,
                               SM_GetSystemFont(),
                               0,
                               False,
                               False,
                               0,
                               &cindex);
    SM_SetGadgetBackcolor(forelabel, IconData->drawcolor);

}




/***************************************************************************
**
** IconExit
*/
int IconExit(WindowType *w)
{
    IconEditData    *IconEdit;
    char            msg[100];
    int             answer;

    IconEdit = (IconEditData *)SM_GetUserData(w);
    if (IconEdit->modified) {
        sprintf(msg, "Pixmap has been modified. Save changes before quitting?");
        answer = SM_YesNoCancelDialog(msg, "Save First", "Quit Now", "Cancel");
        if (answer < 0) {
            return 1;
        }
        else if (answer == 0) {
            if (SavePixmap(w)) {
                return 0;
            }
        }
        SM_DestroyPixmap(IconEdit->pm);
    }
    SM_Exit(NULL);
}


/*****************************************************************************
**
** SetWindowSize
**
** Change the window size to accomodate new pixmaps
*/
void SetWindowSize(WindowType *w)
{
    IconEditData    *icondata;
    RectType        pmr;

    icondata = (IconEditData *)SM_GetUserData(w);

    pmr.Xmin = 67;
    pmr.Ymin = 5;
    pmr.Ymax = pmr.Ymin + icondata->isizex*fatbitsize + 2*SM_GetDefaultBevelDepth();
    pmr.Xmax = pmr.Xmin + icondata->isizex*fatbitsize + 2*SM_GetDefaultBevelDepth();
    GR_SetCanvasRect(w, &pmr);
}



/*****************************************************************************
**
** DrawEditWin
**
** This is the function that draws the entire window, when necessary (this
** is the main window's draw procedure, in other words).
*/
int DrawEditWin(WindowType *w)
{
    int             x,y;
    int             i;
    ColorType       c;
    IconEditData    *ie;

    /*
    ** Get the grid size from the window
    */
    ie = (IconEditData *)SM_GetUserData(w);

    /*
    ** Store the current drawing color before drawing dark gray lines
    */
    memcpy(&c, GR_GetDrawColor(w), sizeof(ColorType));
    GR_SetDrawColor(w, SM_GetSystemColor(SMDARKGRAY));

    /*
    ** Set graphics blocking for  faster drawing, but only if this is
    ** the focus window.
    */
    if (SM_IsFocus(w)) {
        GR_SetBlocking(w);
    }

    /*
    ** Draw the grid
    */
    x = y = fatbitsize-1;
    for (i = 0; i < ie->isizex-1; i++) {
        GR_MoveTo(w,x,0);
        GR_LineTo(w,x,GR_GetCanvasDepth(w));
        x += fatbitsize;
    }
    for (i = 0; i < ie->isizey-1; i++) {
        GR_MoveTo(w,0,y);
        GR_LineTo(w,GR_GetCanvasWidth(w),y);
        y += fatbitsize;
    }

    /*
    ** IMPORTANT: Turn blocking off if it was originally on
    */
    if (SM_IsFocus(w)) {
        GR_UnsetBlocking();
    }

    /*
    ** Draw the pixmap in fatbit style
    */
    DrawFatbitsFromPixmap(w, ie);

    /*
    ** Restore the original drawing color
    */
    GR_SetDrawColor(w, &c);
    return 0;
}


/**************************************************************************
**
** UndoPixmap
**
** Restore the screen pixmap from the undo pixmap
*/
int UndoPixmap(WindowType *w)
{
    IconEditData    *IconData;

    IconData = (IconEditData *)SM_GetUserData(w);
    SM_CopyPixmap(IconData->pm, undopm);
    DrawFatbitsFromPixmap(w, IconData);
    SM_RefreshGadgets(w);
    return 0;
}



/****************************************************************************
**
** DrawFatbitsFromPixmap
*/
void DrawFatbitsFromPixmap(WindowType *w, IconEditData *ie)
{
    int         x, y, blocked = False;
    ColorType   pmcolor;

    /*
    ** Restore the pixmap in the large grid
    */
    if (SM_IsFocus(w)) {
        GR_SetBlocking(w);
        blocked = True;
    }
    for (x = 0; x < ie->isizex; x++) {
        for (y = 0; y < ie->isizey; y++) {
            SM_GetPixmapPixel(ie->pm,x,y,&pmcolor);
            SetFatbit(w, ie->pm, x, y, &pmcolor);
        }
    }
    if (blocked) {
        GR_UnsetBlocking();
    }
}



/****************************************************************************
**
** IconDraw
**
** This is the editor's window procedure. If the user presses a mouse button
** inside the window.
*/
int IconDraw(WindowType *w, EventType *ev)
{
    int         state, x, y;
    RectType    r;
    int         oldx = 0, oldy = 0;
    IconEditData    *IconData;

    /*
    ** We know if we get here that the window is the focus window, but we
    ** don't know if the event occured in this window.  We can check it...
    */
    if ((ev->ButtonEvent == LEFTBUTTONPRESS) && (ev->Region == CANVAS_REGION)) {
        IconData = (IconEditData *)SM_GetUserData(w);
        /*
        ** Copy the current pixmap so we can undo whatever the user does
        ** this time
        */
        SM_CopyPixmap(undopm, IconData->pm);
        IconData->modified = True;
        switch (IconData->tool) {
            case LINETOOL : DrawLine(w, IconData); break;
            case BOXTOOL  : DrawBox(w, IconData); break;
            case CIRCLETOOL  : DrawCircle(w, IconData); break;
            case PENTOOL :
                /*
                ** Trap the mouse here. As long as the mouse button is pressed and
                ** the mouse is inside the cursor keep drawing inside the canvas.
                */
                GR_SetDrawColor(w, IconData->drawcolor);
                GR_GetMouse(w, &state, &x, &y);
                while (state & LEFTBUTTONACTIVE) {
/*
                    r.Xmin = x - (x % fatbitsize) + 1;
                    r.Ymin = y - (y % fatbitsize) + 1;
*/
                    r.Xmin = x - (x % fatbitsize);
                    r.Ymin = y - (y % fatbitsize);
                    if ((r.Xmin != oldx) || (r.Ymin != oldy)) {
                        r.Xmax = r.Xmin + fatbitsize - 1;
                        r.Ymax = r.Ymin + fatbitsize - 1;
                        GR_DrawSolidRect(w, &r);
                        oldx = r.Xmin;
                        oldy = r.Ymin;
                        /*
                        ** Set the icon pixmap also
                        */
                        SM_SetPixmapPixel(IconData->pm,
                                          x/fatbitsize,
                                          y/fatbitsize,
                                          IconData->drawcolor);
                    }
                    GR_GetMouse(w, &state, &x, &y);
                }
                break;
        }
        /*
        ** This value (True) will be returned to the event handler, and
        ** eventually returned to the event loop through SM_ProcessEvent in
        ** the main function
        */
        return True;
    }
    /*
    ** No other actions are relevant if the mouse button is pressed in the
    ** window content region. Return the type of event. This will eventually
    ** get passed back through the SM_ProcessEvent function in Main.
    */
    return ev->Type;
}



/*****************************************************************************
**
** DrawLine
**
** If the line tool is selected, this function is called by the window
** procedure.  The user drags a "rubber-band" line to specify the line
** endpoints.  Then the fatbits are filled in along the line.
*/
void DrawLine(WindowType *w, IconEditData *IconData)
{
    int         state, x, y;
    int         oldx, oldy;
    int         orgx = 0, orgy = 0;
    double      nexty, slope;
    ColorType   oldcolor;
    int         dy;

    /*
    ** Trap the mouse here. As long as the mouse button is pressed and
    ** the mouse is inside the cursor keep drawing inside the canvas.
    */
    GR_GetMouse(w, &state, &orgx, &orgy);
    GR_SetDrawMode(w, SMXOR);
    memcpy(&oldcolor, GR_GetDrawColor(w), sizeof(ColorType));
    GR_SetDrawColor(w, SM_GetSystemColor(SMWHITE));
    oldx = orgx;
    oldy = orgy;
    GR_DrawLine(w, orgx, orgy, oldx, oldy);
    while (state & LEFTBUTTONACTIVE) {
        GR_GetMouse(w, &state, &x, &y);
        if ((x != oldx) || (y != oldy)) {
            GR_DrawLine(w, orgx, orgy, oldx, oldy);
            GR_DrawLine(w, orgx, orgy, x, y);
            oldx = x;
            oldy = y;
        }
    }
    GR_DrawLine(w, orgx, orgy, x, y);
    GR_SetDrawMode(w, SMREP);

    /*
    ** Find all the fatbit pixels on the path and light them up.
    ** First sort endpoints in X-ascending order
    */
    if (x < orgx) {
        oldx = orgx;
        orgx = x;
        oldy = orgy;
        orgy = y;
    }

    /*
    ** Find the slope of the line
    */
    if ((orgy - oldy) == 0) {
        slope = 9999;
    }
    else {
        slope = ((double)oldx - (double)orgx) / ((double)oldy - (double)orgy);
        dy = (slope < 0) ? -1 : 1;
        slope = fabs(slope);
    }

    /*
    ** Transform the graphics canvas points to pixmap points
    */
    orgx /= fatbitsize;
    orgy /= fatbitsize;
    oldx /= fatbitsize;
    oldy /= fatbitsize;
    y = orgy;

    /*
    ** Draw the line in the pixmap
    */
    nexty = -1;
    GR_SetDrawColor(w,&oldcolor);
    if (orgx == oldx) {
        if (orgy < oldy) {
            for (y = orgy; y <= oldy; y++) {
                SM_SetPixmapPixel(IconData->pm, orgx, y, IconData->drawcolor);
                SetFatbit(w, IconData->pm, orgx, y, IconData->drawcolor);
            }
        }
        else {
            for (y = oldy; y < orgy; y++) {
                SM_SetPixmapPixel(IconData->pm, orgx, y, IconData->drawcolor);
                SetFatbit(w, IconData->pm, orgx, y, IconData->drawcolor);
            }
        }
    }
    else if (orgy == oldy) {
        for (x = orgx; x <= oldx; x++) {
            SM_SetPixmapPixel(IconData->pm, x, orgy, IconData->drawcolor);
            SetFatbit(w, IconData->pm, x, orgy, IconData->drawcolor);
        }
    }
    else {
        for (x = orgx; x <= oldx; x++) {
            SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
            SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
            nexty += 1.0/slope;
            if (nexty >= 0.0) {
                y += dy;
                nexty -=1.0;
                while (nexty > 0.0) {
                    SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
                    SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
                    nexty -=1.0;
                    y += dy;
                    if (y == oldy) {
                        break;
                    }
                }
            }
        }
    }
/*
    DrawFatbitsFromPixmap(w, IconData);
*/
    GR_SetDrawColor(w,&oldcolor);
}



/*****************************************************************************
**
** DrawBox
**
** Called by the window procedure if the Box tool is active.  The user
** stretches out a rubber-band box using the mouse.  When the user lets go
** of the mouse button, the code fills in the fatbits.
*/
void DrawBox(WindowType *w, IconEditData *IconData)
{
    int         state, x, y;
    RectType    r, oldr;
    int         oldx, oldy;
    int         orgx, orgy;
    ColorType   oldcolor;

    /*
    ** Trap the mouse here. As long as the mouse button is pressed and
    ** the mouse is inside the cursor keep drawing inside the canvas.
    */
    GR_GetMouse(w, &state, &oldr.Xmin, &oldr.Ymin);
    oldr.Xmax = orgx = oldx = oldr.Xmin;
    oldr.Ymax = orgy = oldy = oldr.Ymin;
    GR_SetDrawMode(w, SMXOR);
    memcpy(&oldcolor, GR_GetDrawColor(w), sizeof(ColorType));
    memcpy(&r, &oldr, sizeof(RectType));
    GR_SetDrawColor(w, SM_GetSystemColor(SMWHITE));
    GR_DrawRect(w, &oldr);
    while (state & LEFTBUTTONACTIVE) {
        GR_GetMouse(w, &state, &x, &y);
        if ((x != oldx) || (y != oldy)) {
            if (x < orgx) {
                r.Xmin = x;
                r.Xmax = orgx;
            }
            else {
                r.Xmax = x;
                r.Xmin = orgx;
            }
            if (y < orgy) {
                r.Ymax = orgy;
                r.Ymin = y;
            }
            else {
                r.Ymax = y;
                r.Ymin = orgy;
            }
            GR_DrawRect(w, &oldr);
            GR_DrawRect(w, &r);
            oldx = x;
            oldy = y;
            memcpy(&oldr, &r, sizeof(RectType));
        }
    }
    GR_DrawRect(w, &oldr);
    GR_SetDrawMode(w, SMREP);

    oldr.Xmin /= fatbitsize;
    oldr.Xmax /= fatbitsize;
    oldr.Ymax /= fatbitsize;
    oldr.Ymin /= fatbitsize;

    for (x = oldr.Xmin, y = oldr.Ymin; x < oldr.Xmax; x++) {
        SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
        SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
    }
    for (x = oldr.Xmax, y = oldr.Ymin; y < oldr.Ymax; y++) {
        SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
        SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
    }
    for (x = oldr.Xmax, y = oldr.Ymax; x > oldr.Xmin; x--) {
        SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
        SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
    }
    for (x = oldr.Xmin, y = oldr.Ymax; y > oldr.Ymin; y--) {
        SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
        SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
    }
}



/*****************************************************************************
**
** DrawCircle
**
** Called by the window procedure if the Circle tool is on.  The user
** provides the bounding box of a circle using a "rubber-band" box.  The
** code then does a (VERY SLOW) rendering of the circle in the fatbit region.
*/
void DrawCircle(WindowType *w, IconEditData *IconData)
{
    int         state, x, y, cx, cy, radx, rady;
    RectType    r, oldr;
    int         oldx, oldy;
    int         orgx = 0, orgy = 0;
    double      nexty, slope;
    ColorType   oldcolor;

    /*
    ** Trap the mouse here. As long as the mouse button is pressed and
    ** the mouse is inside the cursor keep drawing inside the canvas.
    */
    GR_GetMouse(w, &state, &oldr.Xmin, &oldr.Ymin);
    /*
    ** Set up the initial (probably degenerate) rectangle
    */
    oldr.Xmax = orgx = oldx = oldr.Xmin;
    oldr.Ymax = orgy = oldy = oldr.Ymin;
    memcpy(&r, &oldr, sizeof(RectType));

    GR_SetDrawMode(w, SMXOR);

    /*
    ** Save the current drawing color; we'll restore it later
    */
    memcpy(&oldcolor, GR_GetDrawColor(w), sizeof(ColorType));
    GR_SetDrawColor(w, SM_GetSystemColor(SMWHITE));

    /*
    ** Center the circle inside the rectangle dragged by the user.  The
    ** circle radius is half the rectangle side; use the shortest side as
    ** the radius so it's always inside the rect.
    */
    radx = (oldr.Xmax - oldr.Xmin)/2;
    rady = (oldr.Ymax - oldr.Ymin)/2;
    cx = oldr.Xmin + radx;
    cy = oldr.Ymin + rady;
    GR_DrawRect(w, &oldr);
    if (rady < radx) {
        radx = rady;
    }
    GR_DrawCircle(w, cx, cy, radx);
    while (state & LEFTBUTTONACTIVE) {
        GR_GetMouse(w, &state, &x, &y);
        if ((x != oldx) || (y != oldy)) {
            if (x < orgx) {
                r.Xmin = x;
                r.Xmax = orgx;
            }
            else {
                r.Xmax = x;
                r.Xmin = orgx;
            }
            if (y < orgy) {
                r.Ymax = orgy;
                r.Ymin = y;
            }
            else {
                r.Ymax = y;
                r.Ymin = orgy;
            }
            radx = (oldr.Xmax - oldr.Xmin)/2;
            rady = (oldr.Ymax - oldr.Ymin)/2;
            cx = oldr.Xmin + radx;
            cy = oldr.Ymin + rady;
            if (rady < radx) {
                radx = rady;
            }
            GR_DrawRect(w, &oldr);
            GR_DrawCircle(w, cx, cy, radx);
            radx = (r.Xmax - r.Xmin) / 2;
            rady = (r.Ymax - r.Ymin) / 2;
            cx = r.Xmin + radx;
            cy = r.Ymin + rady;
            if (rady < radx) {
                radx = rady;
            }
            GR_DrawRect(w, &r);
            GR_DrawCircle(w, cx, cy, radx);
            oldx = x;
            oldy = y;
            memcpy(&oldr, &r, sizeof(RectType));
        }
    }
    radx = (oldr.Xmax - oldr.Xmin)/2;
    rady = (oldr.Ymax - oldr.Ymin)/2;
    cx = oldr.Xmin + radx;
    cy = oldr.Ymin + rady;
    GR_DrawRect(w, &oldr);
    if (rady < radx) {
        radx = rady;
    }
    GR_DrawCircle(w, cx, cy, radx);
    GR_SetDrawMode(w, SMREP);
    cx /= fatbitsize;
    cy /= fatbitsize;
    radx /= fatbitsize;
    slope = 6.283;
    if (radx) {
        for (nexty = 0.0; nexty < slope; nexty += slope/(slope*radx*2)) {
            x = (int)((double)radx * cos(nexty) + cx + 0.5);
            y = (int)((double)radx * sin(nexty) + cy + 0.5);
            if ((x < 0) || (x > IconData->isizex) || (y < 0) || (y > IconData->isizey)) {
                continue;
            }
            SM_SetPixmapPixel(IconData->pm, x, y, IconData->drawcolor);
            SetFatbit(w, IconData->pm, x, y, IconData->drawcolor);
        }
    }
}




/*****************************************************************************
**
** SetFatbit
**
*/
void SetFatbit(WindowType *w, PixmapType *pm, int x, int y, ColorType *c)
{
    RectType    r;

    r.Xmin = x*fatbitsize;
    r.Ymin = y*fatbitsize;
    r.Xmax = r.Xmin + fatbitsize - 1;
    r.Ymax = r.Ymin + fatbitsize - 1;
    GR_SetDrawColor(w, c);
    GR_DrawSolidRect(w, &r);
}



/******************************************************************************
**
** About
**
** Displays an "about" dialog box.
*/
int About(WindowType *w)
{
    WindowType  *about;
    RectType    rt;

    rt.Xmin = 50;
    rt.Ymin = 175;
    rt.Xmax = rt.Xmin + 400;
    rt.Ymax = rt.Ymin + 175;
    about = SM_NewWindow(&rt, "About IconEdit", DIALOG | NOBACKING, NULL, NULL);
    rt.Xmin = 1;
    rt.Xmax = -999;
    rt.Ymin = -200;
    rt.Ymax = -275;
    SM_CreateLabel(about,&rt,"ICONEDIT",NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,False,False,False,NULL);
    rt.Ymin = rt.Ymax - 10;
    rt.Ymax = rt.Ymin - 75;
    SM_CreateLabel(about, &rt,
                   "Icon Editor for the TWS Window System",
                   NULL, 
                   SM_GetSystemFont(),
                   ALIGNCENTER, False, False, False,NULL);
    rt.Ymin = rt.Ymax - 175;
    rt.Ymax = rt.Ymin - 75;
    SM_CreateLabel(about, &rt,
                   "Release 4.1",
                   NULL, 
                   SM_GetSystemFont(),
                   ALIGNCENTER, False, False, False, NULL);

    rt.Xmin = -400;
    rt.Xmax = -600;
    rt.Ymax = SM_GetContentDepth(about) - 10;
    rt.Ymin = rt.Ymax - 30;
    SM_CreateButton(about,
                    &rt,
                    "OK",
                    NULL,
                    CloseAbout);
    SM_OpenWindow(about);
    return 0;
}


/*****************************************************************************
**
** CloseAbout
**
** Close the About... dialog box
*/
int CloseAbout(ButtonType *b)
{
    SM_CloseWindow(SM_GetGadgetWindow(b));
    return True;
}


/*****************************************************************************
**
** SetDrawColor
**
** Sets the draw color in the code and in the color foreground label
*/
int SetDrawColor(ButtonType *b)
{
    WindowType  *w;
    IconEditData    *IconData;

    cindex = (int)SM_GetButtonData(b);
    w = SM_GetGadgetWindow(b);
    IconData = (IconEditData *)SM_GetUserData(w);
    IconData->drawcolor = SM_GetSystemColor(cindex);
    SM_SetGadgetBackcolor(forelabel, IconData->drawcolor);
    SM_RefreshGadgets(w);
    return 0;
}


/*****************************************************************************
**
** SetBackColor
**
** Sets the drawing background color in the fatbit drawing area, the background
** color label, and the actual size pixmap.  Setting the background color
** immediately redraws all these items.  Pixels that are already the background
** color are toggled to the old background color so they don't disappear.
*/
int SetBackColor(ButtonType *b)
{
    WindowType      *w;
    ColorType       curr_bg, curr_pix, new_bg;
    IconEditData    *IconData;
    int             x, y;
    RectType        r;

    /*
    ** The button data value is the index for the new background color
    */
    bindex = (int)SM_GetButtonData(b);

    /*
    ** Get the button's parent window, which contains the icon data struct
    */
    w = SM_GetGadgetWindow(b);
    IconData = (IconEditData *)SM_GetUserData(w);

    /*
    ** Store the current background color and get the new one from the system
    */
    memcpy(&curr_bg, IconData->backcolor, sizeof(ColorType));
    memcpy(&new_bg, SM_GetSystemColor(bindex), sizeof(ColorType));
    IconData->backcolor = SM_GetSystemColor(bindex);

    GR_SetBlocking(w);
    /*
    ** Now set all current background pixels in pixmap to this color
    */
    for (y = 0; y < IconData->isizey; y++) {
        for (x = 0; x < IconData->isizex; x++) {
            SM_GetPixmapPixel(IconData->pm, x, y, &curr_pix);
            if (SM_IsColorEqual(&curr_pix, &curr_bg)) {
                /*
                ** Change colors in the pixmap that are the old background
                ** color to the new background color
                */
                SM_SetPixmapPixel(IconData->pm, x, y, &new_bg);
                r.Xmin = x*fatbitsize;
                r.Ymin = y*fatbitsize;
                r.Xmax = r.Xmin + fatbitsize - 1;
                r.Ymax = r.Ymin + fatbitsize - 1;
                GR_SetDrawColor(w, &new_bg);
                GR_DrawSolidRect(w, &r);
            }
            else if (SM_IsColorEqual(&curr_pix, &new_bg)) {
                /*
                ** So we don't lose pieces of the drawing that were drawn
                ** in the color that's going to become the background, set
                ** any pixels that are already the new background color to
                ** the old background color
                */
                SM_SetPixmapPixel(IconData->pm, x, y, &curr_bg);
                r.Xmin = x*fatbitsize;
                r.Ymin = y*fatbitsize;
                r.Xmax = r.Xmin + fatbitsize - 1;
                r.Ymax = r.Ymin + fatbitsize - 1;
                GR_SetDrawColor(w, &curr_bg);
                GR_DrawSolidRect(w, &r);
            }
        }
    }
    GR_UnsetBlocking();
    SM_SetGadgetRedrawFlag(forelabel, True);
    SM_SetGadgetBackcolor(backlabel, IconData->backcolor);
    SM_RefreshGadgets(w);
    return 0;
}




/****************************************************************************
**
** ClearPixmap
**
** Set the pixmap back to the graphics background color
*/
int ClearPixmap(WindowType *w)
{
    IconEditData    *IconData;
    int             x, y;
    RectType        r;

    /*
    ** Get the IconData struct from the user data field of the window
    */
    IconData = (IconEditData *)SM_GetUserData(w);

    /*
    ** Set blocking for speedup
    */
    GR_SetBlocking(w);

    /*
    ** Now set the whole pixmap to this color
    */
    for (y = 0; y < IconData->isizey; y++) {
        for (x = 0; x < IconData->isizex; x++) {
            SM_SetPixmapPixel(IconData->pm, x, y, IconData->backcolor);
            r.Xmin = x*fatbitsize;
            r.Ymin = y*fatbitsize;
            r.Xmax = r.Xmin + fatbitsize - 1;
            r.Ymax = r.Ymin + fatbitsize - 1;
            GR_SetDrawColor(w, IconData->backcolor);
            GR_DrawSolidRect(w, &r);
        }
    }
    GR_UnsetBlocking();
    SM_RefreshGadgets(w);
    return 0;
}


/***************************************************************************
**
** WritePixmap
**
** Writes a pixmap to a disk file
*/
int WritePixmap(WindowType *w)
{
    char            *fname;
    char            msg[100];
    FILE            *f;
    int             rsp;
    IconEditData    *icondata;

    icondata = (IconEditData *)SM_GetUserData(w);
    /*
    ** Get a file from the user
    */
    fname = SM_GetFilename("*.icn");
    if (fname == NULL) {
        return 1;
    }
    /*
    ** If the file already exists then warn the user before overwriting
    */
    if ((f = fopen(fname, "r")) != NULL) {
        fclose(f);
        sprintf(msg, "OK to overwrite %s?",fname);
        rsp = SM_YesNoDialog(msg, "OK", "Cancel");
        if (rsp) {
            free(fname);
            return 1;
        }

    }
    /*
    ** Write the pixmap to the file
    */
    if (SM_WritePixmap(icondata->pm, fname)) {
        sprintf("Error writing pixmap file %s",fname);
        SM_OKDialog(msg);
        free(fname);
        return 1;
    }
    icondata->modified = False;
    sprintf(msg, "Iconedit: %s",fname);
    SM_SetWindowTitle(w, msg);
    free(fname);
    return 0;
}


/***************************************************************************
**
** ReadPixmap
**
** Reads a pixmap from disk
*/
int ReadPixmap(WindowType *w)
{
    char            *fname;
    char            msg[100];
    int             rsp;
    IconEditData    *icondata;
    PixmapType      *pm;
    char            file[MAXFILE], ext[MAXEXT];

    icondata = (IconEditData *)SM_GetUserData(w);
    if (icondata->modified) {
        sprintf(msg,"Discard changes to %s",icondata->filename);
        if (SM_YesNoDialog(msg, "Yes", "Cancel")) {
            return 1;
        }
    }
    rsp = 0;
    while (!rsp) {
        /*
        ** Get a file from the user
        */
        fname = SM_GetFilename("*.icn");
        if (fname == NULL) {
            return 1;
        }
        if (icondata->filename) {
            free(icondata->filename);
        }
        fnsplit(fname,NULL, NULL, file, ext);
        icondata->filename = (char *)malloc(strlen(file) + strlen(ext) + 1);
        strcpy(icondata->filename,file);
        strcat(icondata->filename,ext);
        /*
        ** Load the requested pixmap
        */
        pm = SM_ReadPixmap(fname);
        if (pm == NULL) {
            sprintf(msg,"Error reading pixmap file %s",fname);
            rsp = SM_YesNoDialog(msg, "Retry", "Cancel");
            if (rsp) {
                return 1;
            }
        }
        else {
            rsp = 1;
        }
    }

    /*
    ** Check new pixmap size
    */
    if ((SM_GetPixmapWidth(pm) != SM_GetPixmapWidth(icondata->pm))
    || (SM_GetPixmapDepth(pm)) != SM_GetPixmapDepth(icondata->pm)) {
        /*
        ** This pixmap is different size than the current one.  Redraw the
        ** window
        */
        icondata->isizex = SM_GetPixmapWidth(pm);
        icondata->isizey = SM_GetPixmapDepth(pm);
        SetWindowSize(w);
    }
    /*
    ** Replace the current pixmap with the new one.
    */
    SM_DestroyPixmap(icondata->pm);
    icondata->pm = pm;
    SM_MovePixmap(icondata->pm, 74 + icondata->isizex*5, 5);
    /*
    ** Adjust the undo pixmap as necessary...
    */
    SM_DestroyPixmap(undopm);
    undopm = SM_CreatePixmap(NULL, SM_GetGadgetBound(pm), 0);
    SM_CopyPixmap(undopm, icondata->pm);
    SM_SetPixmapBorder(pm, SCULPTED);

    /*
    ** Since this gadget is read from disk it doesn't belong to any window
    ** yet.  This function connects it to a window.
    */
    SM_AttachGadget(w, SM_GetGadgetSuperclass(pm));
    SM_RefreshGadgets(w);
    DrawFatbitsFromPixmap(w, icondata);

    sprintf(msg,"Iconedit: %s",icondata->filename);

    SM_SetWindowTitle(w,msg);
    return 0;
}




/***************************************************************************
**
** SavePixmap
**
** Writes the pixmap to a file
*/
int SavePixmap(WindowType *w)
{
    char            msg[100];
    FILE            *f;
    int             rsp;
    IconEditData    *icondata;

    icondata = (IconEditData *)SM_GetUserData(w);
    if (icondata->filename == NULL) {
        return WritePixmap(w);
    }
    if ((f = fopen(icondata->filename, "r")) == NULL) {
        sprintf(msg, "WARNING : File %s no longer exists! OK to continue writing?",icondata->filename);
        rsp = SM_YesNoDialog(msg, "Continue", "Cancel");
        if (rsp) {
            return 1;
        }
    }
    fclose(f);
    /*
    ** Write the pixmap to the file
    */
    if (SM_WritePixmap(icondata->pm, icondata->filename)) {
        sprintf("Error writing pixmap file %s",icondata->filename);
        SM_OKDialog(msg);
        return 1;
    }
    icondata->modified = False;
    return 0;
}



/***************************************************************************
**
** SetTool
*/
int SetTool(ButtonType *b)
{
    WindowType      *w;
    IconEditData    *icondata;
    int             tool;

    w = SM_GetGadgetWindow(b);
    icondata = (IconEditData *)SM_GetUserData(w);
    tool = (int)SM_GetButtonData(b);
    icondata->tool = tool;
    if (b->pixmap) {
        SM_CopyPixmap(SelectPixmap, b->pixmap);
    }
    return 0;
}



/****************************************************************************
**
** DisplayHelp
**
** Displays a text file containing usage and other information.
*/
int DisplayHelp(WindowType *w)
{
    WindowType  *works;
    RectType    rt;
    TextboxType *tb;

    rt.Xmin = rt.Ymin = 150;
    rt.Xmax = rt.Xmin + 500;
    rt.Ymax = rt.Ymin + 350;
    works = SM_NewWindow(&rt, "iconedit.hlp", DOCUMENT | NOBACKING, NULL, NULL);
    rt.Xmin = rt.Ymin = 1;
    rt.Xmax = SM_GetContentWidth(works) - 2;
    rt.Ymax = SM_GetContentDepth(works) - 2;
    tb = SM_CreateTextbox(works,
                          &rt,
                          "iconedit.hlp",
                          SM_GetSystemFont(),
                          True,
                          True,
                          NULL);
    SM_SetResizeProc(works, ResizeText);
    SM_SetUserData(works, (void *)tb);
    SM_OpenWindow(works);
    return 0;
}


/*****************************************************************************
**
** ResizeText
**
** This function is called if the user resizes the window containing the
** help text.  It resizes the textbox within the window so it continues to fill
** it.  We have to do this because textboxes don't recognize virtual bounds.
*/
int ResizeText(WindowType *w)
{
    TextboxType *tb;
    RectType    rt;

    tb = (TextboxType *)SM_GetUserData(w);
    rt.Xmin = rt.Ymin = 1;
    rt.Xmax = SM_GetContentWidth(w) - 2;
    rt.Ymax = SM_GetContentDepth(w) - 2;
    SM_SetTextboxRect(tb, &rt);
    return 0;
}


/*****************************************************************************
**
** PrntScrn
**
** Writes a PCX file of the whole screen when the user presses Ctrl-PrtScrn
*/
void PrntScrn(WindowType *w, RectType *r, char *fname)
{
    SM_SaveScreenAsPCX(fname);
}


