/*
** imgstuff.c
**
** Contains the gadget callback routines for manipulating the image
**
** Copyright 1993, 1994 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
*/

#include    <stdlib.h>
#include    <stdio.h>
#include    "smwindow.h"
#include    "smgadget.h"
#include    "smcolor.h"
#include    "smslider.h"
#include    "smbutton.h"
#include    "smchkbx.h"
#include    "pcxstuff.h"
#include    "imgstuff.h"

int SetBrightness(SliderType *slider)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(slider);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    /*
    ** Modify the window colors
    */
    img->bright = SM_GetSliderPosition(slider);
    ModifyWindowPalette(w);
    return 0;
}

int SetContrast(SliderType *slider)
{
    WindowType      *w;
    int             i;
    ImageDataType   *img;
    int             divs, cont, inc;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(slider);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    /*
    ** Modify the window colors
    */
    img->contrast = SM_GetSliderPosition(slider);
    divs = abs(img->contrast);
    divs = img->ncolors / (2*divs + 1);
    /*
    ** If contrast is negative, reduce the image contrast, otherwise increase it
    */
    if (img->contrast) {
        cont = img->contrast;
        inc = -(abs(img->contrast) / img->contrast);
    }
    else {
        cont = 0;
        divs = img->ncolors + 1;
    }
    img->contable[0] = -cont;
    for (i = 1; i < img->ncolors; i++) {
        img->contable[i] = -cont;
        if ((i % divs) == 0) {
            cont += inc;
        }
    }
    ModifyWindowPalette(w);
    return 0;
}

/**************************************************************************
**
** SetRedOffset
**
** Sets the Red offset value and adjusts the window color palette
*/
int SetRedOffset(SliderType *slider)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(slider);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    /*
    ** Modify the window colors
    */
    img->roff = SM_GetSliderPosition(slider);
    ModifyWindowPalette(w);
    return 0;
}


/**************************************************************************
**
** SetGreenOffset
**
** Sets the Red offset value and adjusts the window color palette
*/
int SetGreenOffset(SliderType *slider)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(slider);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    /*
    ** Modify the window colors
    */
    img->goff = SM_GetSliderPosition(slider);
    ModifyWindowPalette(w);
    return 0;
}


/**************************************************************************
**
** SetBlueOffset
**
** Sets the Red offset value and adjusts the window color palette
*/
int SetBlueOffset(SliderType *slider)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(slider);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    /*
    ** Modify the window colors
    */
    img->boff = SM_GetSliderPosition(slider);
    ModifyWindowPalette(w);
    return 0;
}


/************************************************************************
**
** ModifyWindowPalette
**
** Modifies a window's color palette based on the image variables.  Called
** by the gadget callback routines that manipulate the image colors,
** brightness, contrast, etc.
*/
void ModifyWindowPalette(WindowType *w)
{
    int             i, j;
    ImageDataType   *img;
    int             r, g, b;
    int             k;

    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    if (img->invert) {
        for (i = SM_nApplicationColors()-1; i >= 0; i--) {
            k = img->rantable[i];
            j = (img->ncolors - k - 1)*3;
            r = img->bright + img->roff + img->contable[i] + img->palette[j]  ;
            g = img->bright + img->goff + img->contable[i] + img->palette[j+1]  ;
            b = img->bright + img->boff + img->contable[i] + img->palette[j+2]  ;
            if (r > 255) r = 255;
            if (g > 255) g = 255;
            if (b > 255) b = 255;
            if (r < 0) r = 0;
            if (g < 0) g = 0;
            if (b < 0) b = 0;
            if (img->mono) {
                r = (int)(0.7 * g + 0.2 * r + 0.1 * b);
                g = (int)(0.7 * g + 0.2 * r + 0.1 * b);
                b = (int)(0.7 * g + 0.2 * r + 0.1 * b);
            }
            SM_ModifyWindowColor(w, i, r, g, b);
        }
    }
    else {
        for (i = 0; i < SM_nApplicationColors(); i++) {
            k = img->rantable[i];
            j = k*3;
            r = img->palette[j] + img->bright + img->roff + img->contable[i];
            g = img->palette[j+1] + img->bright + img->goff + img->contable[i];
            b = img->palette[j+2] + img->bright + img->boff + img->contable[i];
            if (r > 255) r = 255;
            if (g > 255) g = 255;
            if (b > 255) b = 255;
            if (r < 0) r = 0;
            if (g < 0) g = 0;
            if (b < 0) b = 0;
            if (img->mono) {
                r = (int)(0.7 * g + 0.2 * r + 0.1 * b);
                g = (int)(0.7 * g + 0.2 * r + 0.1 * b);
                b = (int)(0.7 * g + 0.2 * r + 0.1 * b);
            }
            SM_ModifyWindowColor(w, i, r, g, b);
        }
    }
}

/****************************************************************************
**
** ResetImage
**
** Restores all color, brightness, etc., to default levels
*/
int ResetImage(ButtonType *button)
{
    WindowType      *w;
    int             i;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(button);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    img->roff = img->boff = img->goff = img->bright = img->contrast
    = img->invert = img->random = img->mono = 0;
    /*
    ** Clear the contrast table
    */
    for (i = 0; i < img->ncolors; i++) {
        img->contable[i] = 0;
    }
    /*
    ** Set the slider values.  Note that changing the gadget state directly
    ** also generates a call to the gadget callback function.  Since each
    ** callback function resets the image color table by calling
    ** ModifyWindowPalette(), that routine needs to be fast!
    */
    SM_SetSliderPosition(img->RedSlider, 0);
    SM_SetSliderPosition(img->GreenSlider, 0);
    SM_SetSliderPosition(img->BlueSlider, 0);
    SM_SetSliderPosition(img->BrightSlider, 0);
    SM_SetSliderPosition(img->ContrastSlider, 0);
    SM_SetCheckboxState(img->InvertButton, False);
    SM_SetCheckboxState(img->RandomButton, False);
    SM_SetCheckboxState(img->MonoButton, False);
    ToggleRandomColor(img->RandomButton);
    return 0;
}



/****************************************************************************
**
** ToggleInvertColors
**
** Restores all color, brightness, etc., to default levels
*/
int ToggleInvertColor(CheckboxType *check)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(check);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    img->invert = SM_GetCheckboxState(check);
    ModifyWindowPalette(w);
    return 0;
}


/****************************************************************************
**
** ToggleRandomColors
**
** Restores all color, brightness, etc., to default levels
*/
int ToggleRandomColor(CheckboxType *check)
{
    WindowType      *w;
    ImageDataType   *img;
    int             i;

    /*
    ** Get the slider's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(check);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    img->random = SM_GetCheckboxState(check);
    for (i = 0; i < img->ncolors; i++) {
        if (img->random) {
            img->rantable[i] = random(SM_nApplicationColors());
        }
        else {
            img->rantable[i] = i;
        }
    }
    ModifyWindowPalette(w);
    return 0;
}



/****************************************************************************
**
** ToggleMono
**
** Sets the monochrome color flag based on the state of the checkbox gadget
*/
int ToggleMono(CheckboxType *check)
{
    WindowType      *w;
    ImageDataType   *img;

    /*
    ** Get the gadget's parent window.  This is the window containing the
    ** image to be modified.
    */
    w = SM_GetGadgetWindow(check);
    /*
    ** The data field of the window contains PCX image struct
    */
    img = (ImageDataType *)SM_GetUserData(w);
    img->mono = SM_GetCheckboxState(check);
    ModifyWindowPalette(w);
    return 0;
}


