#define BADSIG  1
#define BADFILE 2
#define BADBITS 3
#define  OK       False
#define  ERROR    True   
#define  PACKED   0xC0  /* two hi bits of byte */ 
#define  MASK     0x3F  /* lower six bits */

typedef unsigned short  word;

#include    "smwindow.h"
#include    "smslider.h"
#include    "smchkbx.h"
#include    "smpixmap.h"

typedef struct  _PCXFileHDR
{
    unsigned char     dMAN;             /* signature for Image File         */
    unsigned char     dVER;             /* version info                     */
    unsigned char     encoding;         /* currently 1, RLL                 */
    unsigned char     pixbits;          /* bits per pixel                   */
    word     minX;                      /* Image Width (in pixels)          */
    word     minY;                      /* Image Height ( in pixels )       */
    word     maxX;
    word     maxY;
    word     resX;                      /* resolution of device             */
    word     resY;
    unsigned char     clrmap[48];       /* palette info                     */
    unsigned char     mode;             /* usually ignored                  */
    unsigned char     pixPlanes;        /* number of planes involved        */
    word              picBytes;         /* unsigned chars per line in picture        */
    unsigned char     filler[60];       /* pad to 128 unsigned chars, data starts    */
} PCXFileHDR;

/*
** The Image Data struct contains all the information we need to store
** the PCX image. One of these structs will be attached to each image window.
*/

typedef struct {
    int         width, depth;           /* Image width/depth                */
    int         ncolors;                /* Number of colors (8 or 256)      */
    long        size;                   /* File size in unsigned chars               */
    char        *fname;                 /* Name of the image file           */
    FILE        *f;                     /* File pointer                     */
    long        imgstart;               /* Offset to beginning of image     */
    int         roff, goff, boff;       /* Color adjustment offsets         */
    int         contrast;               /* Cheap contrast adjustment        */
    int         bright;                 /* Brightness adjustment            */
    unsigned char palette[256*3];       /* Image palette                    */
    int           contable[256];        /* Contrast table                   */
    int           rantable[256];        /* Random table                     */
    int         invert;                 /* Color reverse                    */
    int         random;                 /* Randomize colors                 */
    int         mono;                   /* If true then make mono palette   */
    double      zoom;
    /*
    ** We're going to store the gadget pointers here too so we can
    ** manipulate them directly (for example, see the ResetImage callback)
    */
    SliderType  *RedSlider, *GreenSlider, *BlueSlider;
    SliderType  *BrightSlider, *ContrastSlider;
    CheckboxType    *InvertButton, *RandomButton, *MonoButton;
    PCXFileHDR      PCXInfo;
    PixmapType      *iconpm;
    int             iconbuilt;
    LabelType       *cursorlabel;
} ImageDataType;



int Open_PCX( char *fname , ImageDataType *img);
int PCX_Get_Row(FILE *datfile, PCXFileHDR *PCXInfo, int rows, unsigned char *line, int init);
void PCXPalette(ImageDataType *img);

