/*
** smcolor.h
**
** Public header file for color manager
**
** Copyright 1992, 1993 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
** Revision history:
**
** Fri  06-05-1992
** Original version
*/

#ifndef SMCOLOR_H
#define SMCOLOR_H

#include <smtypes.h>
#include <smwindow.h>

/*
** Set the number of reserved system colors
*/
#define NSYSTEMCOLORS       16

/*
#define SMBLACK             0
#define SMDARKGRAY          1
#define SMLIGHTGRAY         2
#define SMMEDIUMGRAY        5
#define SMDARKRB            3
#define SMLIGHTRB           4
#define SMDARKYG            5
#define SMLIGHTYG           6
*/
#define SMBLACK             0
#define SMDARKGRAY          1
#define SMMEDIUMGRAY        2
#define SMLIGHTGRAY         3
#define SMDARKRED           4
#define SMLIGHTRED          5
#define SMDARKGREEN         6
#define SMLIGHTGREEN        7
#define SMDARKBLUE          8
#define SMLIGHTBLUE         9
#define SMDARKYELLOW        10
#define SMLIGHTYELLOW       11
#define SMWORKSPACECOLOR    12
#define SMACTIVEWINCOLOR    13
#define SMACTIVEWINHILITE   14
#define SMWHITE             256

/*
** Macros for accessing window system colors
*/
#define SMCLOSESTCOLOR      0x0004
#define SMNOSHARE           0x0002
#define SMSHARE             0x0001
#define SMFLAGDEFAULT       SMSHARE
#define TEXT_COLOR          SMWHITE     /* System color index constants     */

#define INACTIVE_WINDOW_COLOR       0
#define ACTIVE_TITLE_COLOR          1
#define INACTIVE_TITLE_COLOR        2
#define ACTIVE_MENU_COLOR           3
#define INACTIVE_MENU_COLOR         4
#define ACTIVE_MENU_TEXT_COLOR      5
#define INACTIVE_MENU_TEXT_COLOR    6
#define ACTIVE_WINDOW_BORDER_COLOR  7
#define INACTIVE_WINDOW_BORDER_COLOR    8
#define APPLICATION_TITLE_COLOR         9
#define APPLICATION_TITLE_TEXT_COLOR    10
#define DIALOG_CONTENT_COLOR        11
#define DIALOG_BORDER_COLOR         12
#define DIALOG_TEXT_COLOR           13
#define DIALOG_GADGET_COLOR         14

/*
** The 'transparent' color is really a flag to window gadgets and 
** other elements that they should not be filled
*/
#define TRANSPARENT     ((ColorType *)0x0001)

/*
** Color schemes are obsolete now
*/
#define SMWARM          1
#define SMCOOL          2

/*
** Function prototypes
*/
long        SM_ActivatePalette(WindowType *w);
ColorType  *SM_CreateColor(int r, int g, int b);
ColorType  *SM_CreateWindowPalette(WindowType *w, int ncolors);
ColorType  *SM_GetClosestColor(WindowType *w, int r, int g, int b);
ColorType  *SM_GetClosestSystemColor(ColorType *c);
ColorType  *SM_GetPalette(WindowType *w);
ColorType  *SM_GetSystemColor(int i);
ColorType  *SM_GetWindowColor(WindowType *w, int n);
void        SM_InitColor(int r, int g, int b, ColorType *c);
long        SM_InitSystemColors(long ncolors);
int         SM_IsColorEqual(ColorType *c1, ColorType *c2);
int         SM_ModifyWindowColor(WindowType *w, int n, int r, int g, int b);
int         SM_nApplicationColors(void);
long        SM_nSystemColors(void);
int         SM_nColorBits(void);
int         SM_nWindowColors(WindowType *w);
int         SM_SetSystemColor(int n, int r, int g, int b);
ColorType  *SM_SetWindowColor(WindowType *w, int i, int r, int g, int b, int flag);
void        SM_ModifySystemColor(int i, int r, int g, int b);

void        CloseSystemColors(void);


#endif



