/*
** smeventp.h
**
** Private header for event handler
**
** Copyright 1992, 1993 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
** Revision history:
**
** 19 May 1992
** Original version
**
** Tue  2 Jun 92    Dean Clark
** Removed the Metawindow-specific 'event' field from the SM EventType
** structure.  Replaced with individual fields that are essentially the same
** but not dependent on metawindow lib.
*/

#ifndef SMEVENTP_H
#define SMEVENTP_H

#if defined TWS40

/*
** Event record
*/
typedef struct _smevent {
    int     region;                     /* Window region where event occured*/
    unsigned int event_type;            /* Kind of event                    */
    short   ASCII;                      /* ASCII code for keyboard event    */
    short   ScanCode;                   /* Scan code for special key        */
    int     State;                      /* Keyboard states                  */
    int     Buttons;                    /* Button state                     */
    unsigned char Type;                 /* Event type                       */
    int     manage_event;               /* Window management event          */
    int     CursorX, CursorY;           /* Cursor position when event occurd*/
    int     Time;                       /* Time tick when event occured     */
    long    localtime;                  /* Time of day when event occured   */
} EventType;

#else

typedef unsigned int    uint;

/*
** New event record
*/
typedef struct _eventtype {
    uint    Region;                     /* Window region where cursor is    */
    unsigned short Type;                /* Event type (keypress, etc.)      */

    short   ASCII;                      /* ASCII code for keyboard press    */
    short   ScanCode;                   /* Scan code for keyboard press     */
    short   ShiftState;                 /* Keyboard shift state             */

    unsigned short Button;              /* Which mouse button for press/release*/
    unsigned short ButtonState;         /* Mouse button current state       */
    unsigned short ButtonEvent;         /* Convenience button event type    */
    int     CursorX, CursorY;           /* Hardware Cursor position         */
    int     WinX, WinY;                 /* Window content cursor position   */
    int     GraphX, GraphY;             /* Window graph canvas cursor pos   */

    uint    Time;                       /* Sys clock at event               */
    long    localtime;                  /* Time of day clock                */

    void    *EventWin;                  /* Window the mouse is 'in'         */
    uint    EventMsg;                   /* Event return message             */
} EventType;


#endif
#endif

