/*
** smtypes.h
**
** Mostly macro definitions and a few basic data structures for the TWS window
** manager.  Mostly used by intrinsic modules; applications only need to
** include the smwindow.h header, which includes this one too.
**
** Copyright 1992, 1993 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
** Revision history: 
** 9 April 1992
** Original version (copied from previous source)
*/

#ifndef SMTYPES_H
#define SMTYPES_H

#define SMDEFAULT       0               /* Generic default definition       */

#define NOSCROLLATTRIB  0
#define NOBACKING       0               /* Window content backing store?    */
#define DOCUMENT        1               /* Window types                     */
#define DIALOG          (1 << 1)
#define SIMPLE          (1 << 2)
#define ICONONLY        (1 << 3)
#define PANEL           (1 << 4)
#define ICONINIT        (1 << 5)
#define MENUWIN         (1 << 6)
#define BACKING         (1 << 7)
#define SCROLLATTRIB    (1 << 8)

#define SCULPTED        1               /* Window decorations relief types  */
#define FOURD           2
#define FLAT            3
#define BEVELED         4
#define CHISELED        5

#define MENU            16
#define STATIC          8

#define CONTROLROOT     0               /* Control types                    */
#define BUTTON          1               
#define CHECKBOX        2
#define VSCROLLBAR      3
#define LABEL           4
#define EDITSTRING      5
#define PIXARRAY        7
#define HSCROLLBAR      8
#define PICKLIST        9
#define BOXTEXT         10
#define SLIDER          11
#define STRINGLIST      12
#define CHECKBOXGROUP   13
#define TEXTBOX         14
#define DRAGLABEL       15
#define SMSCROLLBAR     16
#define PIXMAP          17
#define ROTATELIST      18
#define STATICBORDER    19
#define HOTREGION       20

#define VERTICAL        0               /* Slider types                     */
#define HORIZONTAL      1

#define APPTITLE            0
#define BORDERWIDTH         1           /* Window default keys              */
#define TITLEDEPTH          8
#define ACTIVETITLEBAR      9
#define ACTIVEBORDER        14
#define INACTIVETITLEBAR    11
#define INACTIVEBORDER      12
#define TITLEFONTCOLOR      13
#define COLORSCHEME         2
#define GRAPHDEVICE         4
#define MOUSEDEVICE         5
#define FONTPATH            3
#define SYSDEVICE           6
#define SYSTEMFONT          7
#define TEXTFONT            8
#define TITLEFONT           9
#define BEVELDEPTH          10
#define FOCUSCOLOR          11
#define TITLECOLOR          12
#define WINDOWLIMITS        13
#define ICONFONT            14
#define ICONPATH            15

#define UP                  1
#define DOWN                2
#define LEFT                3
#define RIGHT               4
#define UPLEFT              5
#define UPRIGHT             6
#define DOWNLEFT            7
#define DOWNRIGHT           8
#define MOVE_REGION         9
#define CLOSE_REGION        10
#define MENU_REGION         11
#define BASE_MENU_REGION    12
#define WINDOW_REGION       13
#define MAXIMIZE_REGION     14
#define CONTENT_REGION      15
#define WORKSPACE_REGION    16
#define DESKTOP             16
#define MINIMIZE_REGION     17
#define CANVAS_REGION       18
#define APP_TITLE_REGION    19
#define HSCROLL_REGION      20
#define HSCROLLUP_REGION    21
#define HSCROLLDOWN_REGION  22
#define HSCROLLBAR_REGION   23
#define VSCROLL_REGION      24
#define VSCROLLUP_REGION    25
#define VSCROLLDOWN_REGION  26
#define VSCROLLBAR_REGION   27

#define ALIGNLEFT           0           /* Text alignment constants         */
#define ALIGNCENTER         1
#define ALIGNRIGHT          2

#define ALIGNBASELINE       0
#define ALIGNBOTTOM         4
#define ALIGNMIDDLE         2
#define ALIGNTOP            3

#define TXTNORMAL           0           /* Text facing constants            */
#define TXTBOLD             1
#define TXTITALIC           2
#define TXTUNDERLINE        4
#define TXTSTRIKEOUT        8
#define TXTPROPORTIONAL   128

#define SAVE_UNDER_LIMIT    15000

#define CR      0x0d
#define ESC     27
#define BS      8

/*
** Define accelerator key codes
*/
#define ALTQ    16
#define ALTW    17
#define ALTE    18
#define ALTR    19
#define ALTT    20
#define ALTY    21
#define ALTU    22
#define ALTI    23
#define ALTO    24
#define ALTP    25
#define ALTA    30
#define ALTS    31
#define ALTD    32
#define ALTF    33
#define ALTG    34
#define ALTH    35
#define ALTJ    36
#define ALTK    37
#define ALTL    38
#define ALTZ    44
#define ALTX    45
#define ALTC    46
#define ALTV    47
#define ALTB    48
#define ALTN    49
#define ALTM    50

#define KF1     59
#define KF2     60
#define KF3     61
#define KF4     62
#define KF5     63
#define KF6     64
#define KF7     65
#define KF8     66
#define KF9     67
#define KF10    68

#define KHOME   71
#define KUP     72
#define KPGUP   73
#define KLEFT   75
#define KRIGHT  77
#define KEND    79
#define KDOWN   80
#define KPGDN   81
#define KINS    82
#define KDEL    83
#define LARR    75
#define RARR    77
#define KEND    79
#define HOME    71
#define KDEL    83
#define KTAB    9
#define SHTAB   15

#define SF1     84
#define SF2     85
#define SF3     86
#define SF4     87
#define SF5     88
#define SF6     89
#define SF7     90
#define SF8     91
#define SF9     92
#define SF10    93

#define CF1     94
#define CF2     95
#define CF3     96
#define CF4     97
#define CF5     98
#define CF6     99
#define CF7     100
#define CF8     101
#define CF9     102
#define CF10    103

#define AF1     104
#define AF2     105
#define AF3     106
#define AF4     107
#define AF5     108
#define AF6     109
#define AF7     110
#define AF8     111
#define AF9     112
#define AF10    113


/*
** Pixel
*/
typedef struct {
    short X, Y;
} PointType;


typedef struct {
    short Xmin, Ymin, Xmax, Ymax;
} RectType;

/*
** Font record; for now just a pointer
*/
typedef struct {
    void    *fontbuf;
    char    *fontname;
    unsigned long   time;
    int     width, height;
    int     face;
} SysFontType;


typedef char   *FontType;

/*
** Index for the system, title, and icon fonts
*/
#define ndxSYSTEMFONT  0
#define ndxTITLEFONT   1
#define ndxICONFONT    2
#define ndxUSERFONT    3
#define ndxMAXFONTS    8

/*
** Color type definition
*/
typedef struct {
    unsigned char r,g,b;                /* RGB value of this color          */
    int index;                          /* Lookup table index for the color */
    void *owner;                        /* Pointer to object that owns this colors */
    int flag;
    int count;
} ColorType;



/*
** Polygon header
*/
typedef struct {                        /* Polygon "header" structure       */
    unsigned short  polyBgn;            /* beginning index                  */
    unsigned short  polyEnd;            /* ending index                     */
    RectType        polyRect;           /* boundry limits                   */
} PolyHead;


/*
** A polygon
*/
typedef struct _poly {
    int         polycnt;
    PolyHead    polyhdr;
    PointType   polypts[7];
    int         pattern;
} PolygonType;


#define True    1
#define False   0

#endif
