/*
** smwindop.h
**
** Private header for windows
**
** Copyright 1994 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
** Revision history:
**
** 12 Dec 1994
** Original version
**
*/

#ifndef SMWINDOP_H
#define SMWINDOP_H

#include    <smtypes.h>
#include    <smmenup.h>
#include    <smgraphp.h>
#include    <smgadgtp.h>
#include    <smscrolp.h>
#include    <smeventp.h>

/*
** The window structure
*/
typedef struct _win {
    /*
    ** Window display regions
    */
    RectType window;                    /* Max extent of complete window    */
    RectType content;                   /* Drawing area                     */
    RectType orig_content;              /* Old content region for max       */
    RectType titlebar;
    RectType stretch_left,              /* Window 'grab' regions            */
             stretch_up,
             stretch_right,
             stretch_down,
             menubutton,
             iconize,
             fullscreen;
    PolygonType upleft,
            upright,
            downleft,
            downright;
    RectType windowmenu;                /* Window menu region               */
    MenuType *menu;                     /* Window menu                      */

    /*
    ** Define cursor styles for window positions
    */
    int     normcursor, movecursor, resizecursor, contentcursor, gscursor;

    int     hoffset, voffset;

    /*
    ** If the window is created with the SCROLL attribute, these will
    ** be created
    */
    struct _win *hscrollbar, *vscrollbar;
    void        *hscroll, *vscroll;

    /*
    ** Minimum width/height for the window
    */
    int     minx, miny;

    char    *title;                     /* Titlebar title                   */
    void    *icon;                      /* Icon version of the window       */
    int     is_icon;                    /* TRUE if window is iconized       */
    int     is_maximized;               /* TRUE if window is maximized      */
    int     icon_only;                  /* If TRUE window can only be an icon*/
    int     type;                       /* Window type                      */
    void    *data;                      /* User-defined data                */

    struct _gadget *gadgets;            /* Window gadget list               */
    struct _gadget *focusgadget;        /* The gadget with the input focus  */

    /*
    ** Window procedures
    */
    void    (*windrawproc)(struct _win *);/* Procedure for drawing win borders*/
    int     (*drawproc)(struct _win *);   /* Procedure for drawing contents   */
    int     (*redrawproc)(struct _win *); /* Procedure for redrawing contents */
    int     (*resizeproc)(struct _win *); /* Procedure for redrawing contents */
    int     (*userproc)(struct _win *, EventType *);
    int     (*closeproc)(struct _win *);  /* Procedure when window closed     */
    int     (*losefocusproc)(struct _win *);
    int     (*getfocusproc)(struct _win *);

    /*
    ** The window graphics state
    */
    GraphStateType *gs;

    /*
    ** Content region colors
    */
    int     ncolors;                    /* Size of the window color table   */
    ColorType *content_color;           /* Color of content region          */
    int     redraw;                     /* TRUE if window needs to be drawn */
    int     locked;                     /* TRUE if window is locked         */
    struct _win *next, *prev;
} WindowType;

typedef WindowType  PanelType;

/*
** Window default structure
*/
typedef struct _def {
    int         border_width;               /* Width of window scale regions*/
    int         title_bar_depth;            /* Width of title bar           */
    int         menu_bar_depth;             /* Width of menu bar            */
    ColorType   *active_title_bar_color;
    ColorType   *active_border_color;
    ColorType   *inactive_title_bar_color;
    ColorType   *inactive_border_color;
    ColorType   *title_font_color;
    ColorType   *active_up_bevel_color;
    ColorType   *active_down_bevel_color;
    ColorType   *inactive_up_bevel_color;
    ColorType   *inactive_down_bevel_color;
    ColorType   *text_color;
    ColorType   *content_color;
    ColorType   *desktop_color;
    ColorType   *menu_color;
    ColorType   *menu_text_color;
    ColorType   *inactive_menu_text_color;
    FontType    system_font;
    FontType    title_font;
    FontType    text_font;
    FontType    icon_font;
    RectType    screen;
    int         menu_spacing;
    long        colors;
    int         white_index;
    int         color_scheme;
    int         truecolor;
    char        *font_dir;
    char        *graph_device;
    char        *mouse_device;
    char        *graph_mode;
    char        *sysdevice;
    char        *app_title;
    char        *icon_dir;
    int         fncntr;                 /* Incremental counter for tmp file */
    int         icon_align;             /* Icon alignment (top, right, etc.)*/
    int         icon_next_pos;          /* Next icon position               */
    int         iconx;                  /* Icon X position                  */
    int         icon_width;             /* Default icon width               */
    int         icon_depth;             /* Default icon depth               */
    int         bevel_depth;            /* 3D effect depth                  */
    int         limit_windows;          /* If True restrict window size     */
    int         scrollbar_width;        /* Pixel width of window scrollbars */
    int         autoredraw;             /* If true gadget changes redrawn   */
} WinDefaultsType;

void    SetWindowScrollbars(WindowType *w);
void    UpdateWindowScrollbars(WindowType *w);


#endif
