/*
** smwindow.h
**
** Public header for windows functionality. Every program that uses the
** window system must include this file
**
** Copyright 1992, 1993 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
**
** Revision history:
**
** Mon  10-19-1992
** Added linewidth field to graphics state struct
**
** 12 Dec 1994
** Reorganized private header files somewhat.  Also (recently) made some
** interface functions into macros for smaller size and faster execution.
**
*/

#ifndef SMWINDOW_H
#define SMWINDOW_H


#include <smtypes.h>                    /* Basic types and constants        */
#include <smmenup.h>                    /* Menu gadget definition           */
#include <smeventp.h>                   /* Event type definitions           */
#include <smwindop.h>
#include <smgraphp.h>
#include <smwindop.h>

/*
** Function prototypes
*/
int         SM_BringToFront(WindowType *w);
int         SM_CloseAllWindows(void);
int         SM_CloseWindow(WindowType *w);
PanelType  *SM_CreatePanel(int size, int location);
WindowType *SM_FocusWindow(void);
int         SM_DoNothing(WindowType *w);
void        SM_DrawWindow(WindowType *w);
void        SM_DrawWindowContent(WindowType *w);
int         SM_DrawControls(WindowType *w);
void        SM_EraseContent(WindowType *w);
void        SM_Exit(WindowType *w);
void        SM_FreeWindow(WindowType *w);
int         SM_GetColorScheme(void);
int         SM_GetDefaultBorderWidth(void);
int         SM_GetDefaultBevelDepth(void);
int         SM_GetDefaultMenubarDepth(void);
int         SM_GetDefaultScrollbarDepth(void);
ColorType  *SM_GetDefaultTextColor(void);
int         SM_GetDefaultTitlebarDepth(void);
int         SM_GetDisplayDepth(void);
int         SM_GetDisplayWidth(void);
ColorType  *SM_GetTextColor(void);
int         SM_Init(void *graph, void *point);
int         SM_IsFocus(WindowType *w);
int         SM_IsRedraw(WindowType *w);
int         SM_nDisplayBits(void);
int         SM_nDisplayPlanes(void);
WindowType *SM_NewWindow(RectType   *size,
                         char       *title,
                         int        type,
                         MenuType   *defaultmenu,
                         void       *data);
WindowType *SM_OpenApplication(char *title, MenuType *menu);
void        SM_OpenWindow(WindowType *w);
int         SM_RedrawWindow(WindowType *w);
int         SM_ResizeWindow(WindowType *w, int dx, int dy);
int         SM_SaveContent(WindowType *win);
void        SM_SetApplicationMenu(MenuType *m);
void        SM_SetApplicationTitle(char *);
void        SM_SetAutoRedrawFlag(int flag);
void        SM_SetContentColor(WindowType *w, ColorType *color);
int         SM_SetContentRect(WindowType *w, RectType *r);

void        SM_SetMenu(WindowType *w, MenuType *m);
void        SM_SetTitle(WindowType *w, char *title);
void        SM_SetWindowTitle(WindowType *w, char *title);
int         SM_StringWidth(char *string, FontType font, int face);
int         SM_WindowExists(WindowType *w);
WindowType *SM_WorkspaceWindow(void);

/*
** API mappings to lower-level functions
*/
#define SM_DrawAppTitle         DrawAppTitle
#define SM_DrawBorder(w)        DrawBorders((w))
#define SM_GetContentColor(w)   ((w)->content_color)
#define SM_GetContentRect(w)    (&(w)->content)
#define SM_GetContentWidth(w)   ((w)->content.Xmax - (w)->content.Xmin + 1)
#define SM_GetContentDepth(w)   ((w)->content.Ymax - (w)->content.Ymin + 1)
#define SM_MoveWindow(w,x,y)    MoveTheWin((w),(x),(y))
#define SM_GetGraphState(w)     ((w)->gs)
#define SM_GetWindowTitle(w)    ((w)->title)
#define SM_GetUserData(w)       ((w)->data)
#define SM_GetWindowData(w)     ((w)->data)
#define SM_GetWindowDepth(w)    ((w)->window.Ymax - (w)->window.Ymin + 1)
#define SM_GetWindowWidth(w)    ((w)->window.Xmax - (w)->window.Xmin + 1)
#define SM_IsMaximized(w)       ((w)->is_maximized)
#define SM_IsMinimized(w)       ((w)->is_icon)
#define SM_SetCloseProc(w,f)    ((w)->closeproc = (f))
#define SM_SetGadgets(w,g)      ((w)->gadgets = (void *)(g))
#define SM_LockWindow(w)        ((w)->locked = 1)
#define SM_SetDrawProc(w,f)     ((w)->drawproc = (f))
#define SM_SetRedrawFlag(w,f)   ((w)->redraw = (f))
#define SM_SetRedrawProc(w,f)   ((w)->redrawproc = (f))
#define SM_SetResizeProc(w,f)   ((w)->resizeproc = (f))
#define SM_SetUserData(w,d)     ((w)->data = (d))
#define SM_SetWindowData(w,d)   ((w)->data = (d))
#define SM_SetWindow(w)         SetWindow((w))
#define SM_SetWindowProc(w,f)   ((w)->userproc = (f))
#define SM_SetUserProc(w,f)     ((w)->userproc = (f))
#define SM_UnlockWindow(w)      ((w)->locked = 0)
#define SM_WindowRegion         WindowRegion
#define SM_SetGainFocusProc(w,f)    ((w)->getfocusproc = (f))
#define SM_SetLoseFocusProc(w,f)    ((w)->losefocusproc = (f))

int         WindowIntersect(WindowType *w);
void        RestoreDesktop(int i, RectType *r1, RectType *r2);
void        SetWindow(WindowType *w);
void        BringToFront(WindowType *w);
int         WindowIntersectRect(WindowType *ww, RectType *r);
void        ShiftWindow(WindowType *w, int x1, int y1);
int         IsObscured(WindowType *w);
void        Unstack(WindowType *w);
int         MoveTheWin(WindowType *w, int tx, int ty);
int         WindowIntersectAll(WindowType *ww);
void        SetDialogWindow(WindowType *w);
void        SetDocumentWindow(WindowType *w);
void        SetSimpleWindow(WindowType *w);
void        PushWindow(WindowType *w);
WindowType *PtInWindow(int x, int y);
long        smGetDefaultColors(void);
void        SM_SetWindowHorizontalScrollProc(WindowType *w, int (*f)());
void        SM_SetWindowVerticalScrollProc(WindowType *w, int (*f)());
int         WinSliderFunction(SliderType *slider);
void        SM_SetWindowScrollbarMinMax(WindowType *w, int which, int min, int max);
void        SM_UnmapWindow(WindowType *w);
void        SM_SetWindowScrollProc(WindowType *w, int which, int (*f)());
void        SM_GetWindowOffsets(WindowType *w, int *h, int *v);
void        SM_SetWindowOffset(WindowType *w, int which, int *offset);
#endif

