/*
** twsimage.c
**
** TWS demo program for image display.
**
** This program reads and displays a PCX image in a window. The image can be
** resized by changing the window size.
**
** This program demonstrates a number of basic features of the TWS system.
**
** Copyright 1993, 1994 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
*/


/*
** Include the appropriate C & TWS header files
*/
#include    <stdio.h>
#include    <stdlib.h>
#include    <dos.h>
#include    <string.h>
#include    <mem.h>
#ifdef __TURBOC__
#include    <alloc.h>
#else
#include    <malloc.h>
#endif

#include    <smwindow.h>
#include    <smevent.h>
#include    <smgraph.h>
#include    <smmenu.h>
#include    <smpickls.h>
#include    <smcolor.h>
#include    <smmenu.h>
#include    <smlabel.h>
#include    <smtext.h>
#include    <smbutton.h>
#include    <smpixmap.h>
#include    <smchkbx.h>
#include    <smicon.h>
#include    <smwutil.h>
#include    <smslider.h>
#include    <smfont.h>
#include    <smcursor.h>

/*
** Headers for support routines for this application
*/
#include    "pcxstuff.h"
#include    "imgstuff.h"

/*
** Declare application functions
*/
void main(int argc, char *argv[]);
char *GetPCXFile(void);
int ScaleImage(WindowType *w);
int LoadImage(char *fname);
int About(WindowType *w);
int HowItWorks(WindowType *w);
int CloseAbout(ButtonType *b);
int ResizeText(WindowType *w);
void PrntScrn(WindowType *w, RectType *r, char *fname);
int LoadFile(WindowType *w);
void MakeColorSliders(WindowType *w, ImageDataType *img);
int ImageExit(WindowType *w);
int WindowProc(WindowType *w, EventType *event);

#ifdef __TURBOC__
extern unsigned _stklen = 16284U;  /* set stack size to 8K */
#endif

extern FILE *ferr;
PixmapType  *iconpixmap;

void main(int argc, char *argv[])
{
    EventType   ev;
    char        *fname;
    int         msg;
    MenuType    *m;

    /*
    ** Initialize the window system, graphics, fonts, etc.
    */
    SM_Init(0, 0);

    /*
    ** Create an application menu
    */
    m = SM_CreateMenu();
    SM_AddMenuItem(m, ACTION_ITEM, "About...", About, NULL);
    SM_AddMenuItem(m, ACTION_ITEM, "Load", LoadFile, NULL);
    SM_AddMenuItem(m, ACTION_ITEM, "Source", HowItWorks, NULL);
    SM_AddMenuItem(m, ACTION_ITEM, "Quit", ImageExit, NULL);

    /*
    ** Open the application
    */
    SM_OpenApplication("TWS Image Display", m);

    /*
    ** Attach a print screen key handler.
    */
    SM_SetPrintscreenProc(NULL, NULL, "twsimage.pcx", PrntScrn);

    /*
    ** Set the hourglass cursor as our 'user' cursor since its the only
    ** extra one we need
    */
    SM_DefineCursor(USERCURSOR, HOURGLASSCURSOR);

    /*
    ** Just loop around waiting for things to happen
    */
    while (True) {
        SM_GetNextEvent(&ev);
        msg = SM_ProcessEvent(&ev);
        if (msg == KEYPRESS) {
            /*
            ** Here's a way to get out of the program without the mouse.
            ** I always put in a line like this in case I've forgotten to
            ** load the mouse driver!
            */
            if (ev.ASCII == 'q') {
                SM_Exit(NULL);
            }
            /*
            ** For any key other than 'q' get the PCX file from the user.
            ** If the user doesn't enter a valid file then just keep going.
            ** Of course this isn't the normal way the user would open an
            ** image file -- usually he would select from the menu.  This
            ** is just a 'shortcut'
            */
            if ((fname = GetPCXFile()) == NULL) {
                continue;
            }

            /*
            ** Open a window and load the PCX file.  If we somehow fail to
            ** get an image, just keep going.  In the real world you'd
            ** probably display an error first...
            */
            if (LoadImage(fname)) {
                continue;
            }
        }
    }
}


int ImageExit(WindowType *w)
{
    SM_Exit(NULL);
}


/****************************************************************************
**
** LoadFile
**
** This is the function called when the 'Load' application menu item is
** selected.
*/
int LoadFile(WindowType *w)
{
    char    *fname;

    fname = GetPCXFile();
    if (fname == NULL) return 1;
    return LoadImage(fname);
}


/**************************************************************************
**
** GetPCXFile
**
** Displays all the .PCX files in the current directory in a 'Picklist',
** returns the string selected by the user, or NULL if Cancel is selected.
**
*/
char *GetPCXFile(void)
{
    char        **list;                 /* List of PCX file names           */
    int         msg;                    /* Selected PCX file index          */
    int         nfiles;                 /* Number of PCX files in currdir   */
    struct find_t fdat;                 /* DOS file structure               */
    unsigned    code;                   /* DOS file search return code      */
    RectType    r;                      /* Picklist rectangle               */
    char        *fsel;                  /* Name of file selected            */

    nfiles = 0;

    /*
    ** First find out how many PCX files are in the current directory
    */
    code = _dos_findfirst("*.pcx", _A_NORMAL, &fdat);
    while (code == 0) {
        nfiles++;
        code = _dos_findnext(&fdat);
    }

    /*
    ** Allocate enough string pointers for each file name
    */
    list = (char **)malloc(nfiles * sizeof(char *));
    if (list == NULL) {
        return NULL;
    }

    /*
    ** Now go back through and load the stringlist with file names
    */
    code = _dos_findfirst("*.pcx", _A_NORMAL, &fdat);
    nfiles = 0;
    while (code == 0) {
        list[nfiles] = strdup(fdat.name);
        nfiles++;
        code = _dos_findnext(&fdat);
    }

    /*
    ** Display the Picklist
    */
    r.Xmin = r.Ymin = 140;
    r.Xmax = r.Xmin + SM_StringWidth("WWWWWWWW.WWW",
                                     SM_GetSystemFont(),
                                     TXTPROPORTIONAL);
    r.Ymax = r.Ymin + 120;
    msg = SM_Picklist("PCX Files",      /* Picklist window title            */
                      &r,               /* Boundary for the strings         */
                      list,             /* Strings to pick from             */
                      nfiles,           /* Number of strings                */
                      0,                /* Index of first string            */
                      0,                /* Which string pre-selected        */
                      0);               /* Destructor should not free list  */

    /*
    ** If the picklist returns a negative number the user chose CANCEL;
    ** otherwise, copy the string from the list position
    */
    if (msg < 0) {
        fsel = NULL;
    }
    else {
        fsel = strdup(list[msg]);
    }

    /*
    ** Now free the string list
    */
    while (nfiles > 0) {
        nfiles--;
        free(list[nfiles]);
    }
    free(list);

    return fsel;

}


/****************************************************************************
**
** LoadImage
**
** Creates a window and loads the image file into it.
*/
int LoadImage(char *fname)
{
    WindowType      *w;
    RectType        r;
    int             i, j;
    ImageDataType   *img;
    char            string[25];

    img = (ImageDataType *)malloc(sizeof(ImageDataType));
    if (img == NULL) {
        return 1;
    }

    /*
    ** Initialize some image variables
    */
    img->roff =
    img->boff =
    img->goff =
    img->bright =
    img->contrast =
    img->invert =
    img->random =
    img->iconbuilt =
    img->mono = 0;

    /*
    ** Open the image file and load the window header
    */
    if (Open_PCX(fname, img)) {
        return 1;
    }

    /*
    ** Clear the contrast and random index tables
    */
    for (i = 0; i < img->ncolors; i++) {
        img->contable[i] = 0;
        img->rantable[i] = i;
    }
    /*
    ** Open the window.  The rectangle 'r' describes the size of the
    ** window *content*, NOT the outer window boundaries.  The TWS code
    ** takes the content dimensions and adds window border, title bar,
    ** menu, etc., around it.  This is usually much more convenient for
    ** the application programmer.  Who wants to try to calculate border
    ** widths (depends on hardware graphics resolution), title bar
    ** depths (depends on the font), menu depths (if any, also depends on
    ** font size), etc., etc.  Let the window manager do it!
    */
    r.Xmin = r.Ymin = 100;
    r.Xmax = r.Xmin + img->width + 70;  /* Add a little for buttons     */
    r.Ymax = r.Ymin + img->depth + 50;  /* Add a little for info labels */
    img->fname = strdup(fname);
    w = SM_NewWindow(&r, img->fname, DOCUMENT | BACKING, NULL, NULL);

    /*
    ** Create a graphics state for the window.  This rectangle is the size
    ** of the graphics canvas, which is where the image will be drawn.
    ** Without a graphics state we wouldn't be able to draw in the window,
    ** only attach user interface gadgets.
    **
    ** A separate graphics canvas area means the application programmer
    ** doesn't have to worry about drawing over his user interface buttons,
    ** lists and so forth, or calculate offsets to avoid them.  All graphics
    ** is relative to and clipped within the graphics canvas (of course,
    ** you can place user interface gadgets over the canvas if you want to).
    */
    r.Xmin = 60;
    r.Ymin = 40;
    /*
    ** The negative number means the canvas is a 'virtual rectangle'.  That
    ** means it automatically grows and shrinks in proportion to changes in
    ** the window size.  Several gadgets can have virtual dimensions.
    ** For a more complete explanation, see the manual.
    */
    r.Ymax = r.Xmax = -992;
    GR_CreateGraphState(w, &r, SCULPTED);

    /*
    ** Set the draw and resize procedure functions. We don't strictly have
    ** to set the resize procedure, since if there's a draw procedure attached
    ** to the window but no resize procedure, the draw procedure gets
    ** called automatically when the window is resized.
    */
    SM_SetDrawProc(w, ScaleImage);
    SM_SetResizeProc(w, ScaleImage);
    SM_SetWindowProc(w, WindowProc);

    /*
    ** Attach the image to the window
    */
    SM_SetUserData(w, (void *)img);

    /*
    ** Load the image palette into the window.  Note that we don't check if
    ** the graphics state is 16-color, 256-color or 32K colors.  It doesn't
    ** matter, because the same code will work in any case.
    */
    SM_CreateWindowPalette(w, img->ncolors);
    for (i = 0; i < img->ncolors; i++) {
        j = i*3;
        SM_SetWindowColor(w,
                          i,
                          img->palette[j],
                          img->palette[j+1],
                          img->palette[j+2],
                          SMSHARE | SMCLOSESTCOLOR);
    }
    /*
    ** Many (but not all) PCX files store the background color in the first
    ** color palette position.  We'll use this trait later...
    */
    GR_SetBackgroundColor(w, SM_GetWindowColor(w, 0));

    /*
    ** Create some labels to display information about the image
    */
    sprintf(string, "%dx%d", img->width, img->depth);
    r.Ymin = 10;
    r.Xmin = 4;
    r.Xmax = r.Xmin
           + SM_StringWidth(string, SM_GetSystemFont(), TXTPROPORTIONAL)
           + 10;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 6;

    SM_CreateLabel(w,
                   &r,
                   string,
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);
    sprintf(string, "%ldk", img->size/1000);
    r.Xmin = r.Xmax + 4;
    r.Xmax = r.Xmin + GR_StringWidth(string) + 10;
    SM_CreateLabel(w,
                   &r,
                   string,
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);
    sprintf(string, "%d colors", img->ncolors);
    r.Xmin = r.Xmax + 4;
    r.Xmax = r.Xmin + GR_StringWidth(string) + 10;
    SM_CreateLabel(w,
                   &r,
                   string,
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);

    /*
    ** Create the brightness control
    */
    r.Xmin = r.Xmax + 5;
    i = r.Xmin;
    r.Xmax = r.Xmin + GR_StringWidth("Contrast") + 6;
    r.Ymin -= 7;
    r.Ymax -= 8;
    j = r.Ymax;
    SM_CreateLabel(w,
                   &r,
                   "Bright",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   False,
                   NULL);
    r.Ymin += 3;
    r.Ymax -= 6;
    r.Xmin = r.Xmax + 5;
    r.Xmax = r.Xmin + 100;
    img->BrightSlider = SM_CreateSlider(w,
                                        &r,
                                        HORIZONTAL,
                                        -48,
                                        49,
                                        1,
                                        0,
                                        NULL,
                                        SetBrightness);
    r.Ymin = j-1;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 5;
    r.Xmin = i;
    r.Xmax = r.Xmin + GR_StringWidth("Contrast") + 6;
    SM_CreateLabel(w,
                   &r,
                   "Contrast",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNRIGHT,
                   False,
                   False,
                   False,
                   NULL);
    r.Ymin += 3;
    r.Ymax -= 6;
    r.Xmin = r.Xmax + 5;
    r.Xmax = r.Xmin + 100;
    img->ContrastSlider = SM_CreateSlider(w,
                                          &r,
                                          HORIZONTAL,
                                          -48,
                                          49,
                                          1,
                                          0,
                                          NULL,
                                          SetContrast);
    /*
    ** Make a label for the cursor position
    */
    SM_SetRectangle(&r,
                    r.Xmax + 10,
                    10,
                    GR_StringWidth("WWWW") + 10,
                    SM_GetSystemFontHeight() + 6);
    img->cursorlabel = SM_CreateLabel(w,
                                      &r,
                                      NULL,
                                      NULL,
                                      SM_GetSystemFont(),
                                      ALIGNCENTER,
                                      False,
                                      False,
                                      SCULPTED,
                                      NULL);

    MakeColorSliders(w, img);

    /*
    ** Invert colors toggle
    */
    r.Xmin = 4;
    r.Xmax = r.Xmin + 53;
    r.Ymin = 165;
    r.Ymax = r.Ymin + 25;
    img->InvertButton = SM_CreateCheckbox(w,
                    &r,
                    "Invert",
                    ALIGNCENTER,
                    False,
                    NULL,
                    ToggleInvertColor);
    SM_SetCheckboxType(img->InvertButton, SMCBOXBUTTON);
    /*
    ** Monochrome toggle
    */
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + 25;
    img->MonoButton = SM_CreateCheckbox(w,
                    &r,
                    "Mono",
                    ALIGNCENTER,
                    False,
                    NULL,
                    ToggleMono);
    SM_SetCheckboxType(img->MonoButton, SMCBOXBUTTON);
    /*
    ** Random colors toggle
    */
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + 25;
    img->RandomButton = SM_CreateCheckbox(w,
                    &r,
                    "Random",
                    ALIGNCENTER,
                    False,
                    NULL,
                    ToggleRandomColor);
    SM_SetCheckboxType(img->RandomButton, SMCBOXBUTTON);
    /*
    ** Reset button
    */
    r.Ymin = r.Ymax + 12;
    r.Ymax = r.Ymin + 25;
    SM_CreateButton(w,
                    &r,
                    "Reset",
                    NULL,
                    ResetImage);

    /*
    ** Open the window, which will load and display the image
    */
    SM_OpenWindow(w);

    /*
    ** Create an icon for this window.  Use the pixmap we read at init
    */
    SM_CreateIcon(w, NULL, img->iconpm);

    return 0;
}


/*****************************************************************************
**
** MakeColorSliders
**
** These controls allow the user to control the 'saturation' of red, green
** and blue in the image.  By moving the slider up, more color is added to
** the display, while moving the slider down subtracts color.
*/
void MakeColorSliders(WindowType *w, ImageDataType *img)
{
    RectType    r;

    r.Xmin = 10;
    r.Xmax = r.Xmin + 12;
    r.Ymin = 40;
    r.Ymax = r.Ymin + 100;
    img->RedSlider = SM_CreateSlider(w,
                    &r,
                    VERTICAL,
                    50,
                    -49,
                    1,
                    0,
                    NULL,
                    SetRedOffset);
    r.Xmin = r.Xmax + 2;
    r.Xmax = r.Xmin + 12;
    r.Ymin = 40;
    r.Ymax = r.Ymin + 100;
    img->GreenSlider = SM_CreateSlider(w,
                    &r,
                    VERTICAL,
                    50,
                    -49,
                    1,
                    0,
                    NULL,
                    SetGreenOffset);
    r.Xmin = r.Xmax + 2;
    r.Xmax = r.Xmin + 12;
    r.Ymin = 40;
    r.Ymax = r.Ymin + 100;
    img->BlueSlider = SM_CreateSlider(w,
                    &r,
                    VERTICAL,
                    50,
                    -49,
                    1,
                    0,
                    NULL,
                    SetBlueOffset);
    /*
    ** Make some labels for the sliders
    */
    r.Xmin = 10;
    r.Xmax = r.Xmin + 14;
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 2;
    SM_CreateLabel(w,
                   &r,
                   "R",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   False,
                   NULL);
    r.Xmin = r.Xmax ;
    r.Xmax = r.Xmin + 14;
    SM_CreateLabel(w,
                   &r,
                   "G",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   False,
                   NULL);
    r.Xmin = r.Xmax ;
    r.Xmax = r.Xmin + 14;
    SM_CreateLabel(w,
                   &r,
                   "B",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   False,
                   NULL);
}


/************************************************************************
**
** ScaleImage
**
** Redraws the image in the window.  The size of the image is based on
** the graphics canvas size relative to the original image size, and
** the graphics canvas size depends on the window size.
**
** The image scaling algorithm is derived from the 'DDA' line drawing
** algorithm.  It allows the image to be scaled to any size without
** resorting to floating point math or interpolation schemes, although
** the image quality isn't quite as good.  It has the advantage of being
** fast and good enough.
**
** Image aspect ratio is preserved in the scaling, and the entire image
** is always visible in the window.
*/
int ScaleImage(WindowType *w)
{
    double  zoomh, zoomv;           /* Horizontal and vertical scale factors*/
    int     dday, ddax;             /* Counters for DDA algorithm       */
    int     iddy, iddx;             /* Counter for icon dda             */
    int     i, j;                   /* Loop counters                    */
    int     x, y, ix, iy;           /* Window canvas pixel coordinates  */
    int     icnx, icny;
    int     izoom;                  /* DDA increment                    */
    int     iconincx, iconincy;     /* Icon DDA increment               */
    ImageDataType   *imgdata;       /* Image data attached to window    */
    unsigned char   *aspline;       /* Image raster buffer              */
    RectType    r;                  /* Pixmap rectangle                 */
    ColorType   color, *scolor;
    int         pmx, pmy;
    ColorType   **array;

    /*
    ** Show the wait cursor to the user
    */
    SM_SetCursor(USERCURSOR);

    /*
    ** Get the image data from the window
    */
    imgdata = (ImageDataType *)SM_GetUserData(w);

    /*
    ** Erase the graphics canvas
    */
    GR_ClearCanvas(w);

    /*
    ** Figure the scale factor in the horizontal and vertical directions
    */
    zoomh = (double)(GR_GetCanvasWidth(w)) / (double)(imgdata->width);
    zoomv = (double)(GR_GetCanvasDepth(w)) / (double)(imgdata->depth);

    /*
    ** Use the smaller of the two so the image stays in the window. This is
    ** not required; you could also use the larger scale, or the average.
    */
    if (zoomv < zoomh) zoomh = zoomv;
    imgdata->zoom = zoomh;

    /*
    ** izoom is the DDA differential. We use an integer so we avoid
    ** floating point math, which ought to make it run a little faster
    */
    izoom = (int)(1.0/zoomh * 1000);

    /*
    ** If we were *SURE* that the program would only be run on 256-color
    ** graphics hardware, we could use the much faster GR_DrawArray()
    ** graphics function instead of the GR_DrawPoint() function.  However,
    ** at the moment GR_DrawArray does NOT work correctly in either
    ** 16-color or 32K color modes.  The commented-out statements show
    ** how the program would be modified to use GR_DrawArray() instead.
    */
    if (SM_nSystemColors() ==256) {
        array = (ColorType **)malloc((int)(zoomv*imgdata->width*sizeof(ColorType *))+1);
    }

    /*
    ** Position the file pointer to the beginning of image data
    */
    fseek(imgdata->f, imgdata->imgstart, SEEK_SET);

    /*
    ** Allocate a buffer for one raster from the image file.  Quit the
    ** program if the allocation fails.
    */
    aspline = (unsigned char *)malloc(imgdata->width+2);
    if (aspline == NULL) {
        SM_Exit(NULL);
    }
    dday = 0;
    iddy = 0;
    x = y = 0;
    icnx = icny = 0;

    /*
    ** Turn on blocking to speed drawing somewhat
    */
    GR_SetBlocking(w);

    /*
    ** If the icon pixmap hasn't been built yet then build it now
    */
    if (imgdata->iconbuilt == False) {
        r.Xmin = r.Ymin = 0;
        if (SM_GetDisplayWidth() > 800) {
            r.Xmax = 79;
            r.Ymax = (int)(79L * (long)imgdata->depth / (long)imgdata->width);
            if (r.Ymax > 79) r.Ymax = 79;
        }
        else {
            r.Xmax = 63;
            r.Ymax = 63 * imgdata->depth / imgdata->width;
            if (r.Ymax > 63) r.Ymax = 63;
        }
        iconincx = (int)((double)imgdata->width / (double)r.Xmax * 1000.0);
        iconincy = (int)((double)imgdata->depth / (double)r.Ymax * 1000.0);
        imgdata->iconpm = SM_CreatePixmap(NULL, &r, False);
    }

    /*
    ** Initialize the raster line reader
    */
    PCX_Get_Row(imgdata->f, &imgdata->PCXInfo, imgdata->width,aspline,1);

    for (i = 1; i < imgdata->depth; i++) {
        /*
        ** Get one raster of the image.
        */
        PCX_Get_Row(imgdata->f, &imgdata->PCXInfo, imgdata->width,aspline,0);

        pmx = pmy = 0;
        /*
        ** Pixmap icon, if necessary
        */
        if (imgdata->iconbuilt == False) {
            iddy -= 1000;
            iddx = 0;
            while (iddy < 0) {
                for (pmx = 0; pmx < imgdata->width; pmx++) {
                    iddx -= 1000;
                    while (iddx < 0) {
                        memcpy(&color, SM_GetWindowColor(w, aspline[pmx]), sizeof(ColorType));
/*
                        pmy = color.r * 0.2 + color.g * 0.7 + color.b * 0.1;
                        color.r = color.g = color.b = pmy;
*/
                        scolor = SM_GetClosestSystemColor(&color);
                        SM_SetPixmapPixel(imgdata->iconpm, icnx, icny, scolor);
                        icnx++;
                        iddx += iconincx;
                    }
                }
                iddy += iconincy;
                icny++;
                icnx = 0;
            }
        }


        dday -= 1000;

        while (dday < 0) {
            x = 0;
            /*
            ** As long as dday is less than 0, we haven't advanced to the
            ** next image pixel vertically -- that is, we repeat the same
            ** scan line from the image as many times as it takes to reach
            ** the DDA threshold of 0.
            **
            ** Reset the horizontal DDA sample counter. This ensures that
            ** all rasters start at the same place.
            */
            if (SM_nSystemColors() ==256) {
                setmem(array, zoomv*imgdata->width*sizeof(ColorType *)+1, 0x00);
            }
            ddax = 0;
            for (j = 0; j < imgdata->width; j++) {
                ddax -= 1000;
                /*
                ** Same deal as above -- keep repeating the same image
                ** pixel in adjacent window pixels until the DDA counter
                ** crosses the threshold
                */
                while (ddax < 0) {
                    /*
                    ** Don't bother to draw the pixel if it's the same
                    ** color as the background
                    */
                    if (aspline[j] > 0) {
                        if (SM_nSystemColors() ==256) {
                            array[x] = SM_GetWindowColor(w, aspline[j]);
                        }
                        else {
                            GR_SetDrawColor(w, SM_GetWindowColor(w, aspline[j]));
                            GR_DrawPoint(w, x, y);
                        }
                    }
                    x++;
                    /*
                    ** If x is off the right edge of the window then no
                    ** need to continue
                    */
                    if (x > GR_GetCanvasWidth(w)) {
                        j = imgdata->width;
                    }
                    ddax += izoom;
                }
            }

            if (SM_nSystemColors() ==256) {
                GR_DrawArray(w, 0, y, x-1, array);
            }

            dday += izoom;
            y++;
            /*
            ** Test whether the y ordinate is still within the window
            ** boundary, exit if it's out
            */
            if (y > GR_GetCanvasDepth(w)) {
                free(aspline);
                if (SM_nSystemColors() ==256) {
                    free(array);
                }

                /*
                ** Always unset blocking!!!
                */
                GR_UnsetBlocking();
                imgdata->iconbuilt = True;
                SM_SetCursor(MAINCURSOR);
                return 0;
            }
        }
    }

    /*
    ** Always unset blocking!!!
    */
    GR_UnsetBlocking();
    free(aspline);
    imgdata->iconbuilt = True;
    if (SM_nSystemColors() ==256) {
        free(array);
    }
    SM_SetCursor(MAINCURSOR);
    return 0;
}




int About(WindowType *w)
{
    WindowType  *about;
    RectType    rt;

    rt.Xmin = 50;
    rt.Ymin = 175;
    rt.Xmax = rt.Xmin + 400;
    rt.Ymax = rt.Ymin + 225;
    about = SM_NewWindow(&rt, "About twsimage", DIALOG | NOBACKING, NULL, NULL);

    rt.Xmin = rt.Ymin = 2;
    rt.Xmax = SM_GetContentWidth(about) - 4;
    rt.Ymax = SM_GetContentDepth(about) - 45;

    SM_CreateTextbox(about,
                     &rt,
                     "readme",
//                     SM_GetSystemFont(),
                     "hel10.fnt",
                     True,
                     True,
                     NULL);

    rt.Xmin = -400;
    rt.Xmax = -600;
    rt.Ymax = SM_GetContentDepth(about) - 10;
    rt.Ymin = rt.Ymax - 30;
    SM_CreateButton(about,
                    &rt,
                    "OK",
                    NULL,
                    CloseAbout);
    SM_OpenWindow(about);
    return 0;
}


int CloseAbout(ButtonType *b)
{
    SM_CloseWindow(SM_GetGadgetWindow(b));
    return True;
}


int HowItWorks(WindowType *w)
{
    WindowType  *works;
    RectType    rt;
    TextboxType *tb;

    rt.Xmin = 10;
    rt.Ymin = 150;
    rt.Xmax = rt.Xmin + 500;
    rt.Ymax = rt.Ymin + 350;
    works = SM_NewWindow(&rt, "twsimage.c", DOCUMENT | NOBACKING, NULL, NULL);
    rt.Xmin = rt.Ymin = 1;
    rt.Xmax = SM_GetContentWidth(works) - 2;
    rt.Ymax = SM_GetContentDepth(works) - 2;
    tb = SM_CreateTextbox(works,
                          &rt,
                          "twsimage.c",
                          "cour9c.fnt",
                          1,
                          False,        /* Don't word wrap source code      */
                          NULL);
    SM_SetResizeProc(works, ResizeText);
    SM_SetUserData(works, (void *)tb);
    SM_OpenWindow(works);
    return 0;
}


int ResizeText(WindowType *w)
{
    TextboxType *tb;
    RectType    rt;

    tb = (TextboxType *)SM_GetUserData(w);
    rt.Xmin = rt.Ymin = 1;
    rt.Xmax = SM_GetContentWidth(w) - 2;
    rt.Ymax = SM_GetContentDepth(w) - 2;
    SM_SetTextboxRect(tb, &rt);
    return 0;
}

void PrntScrn(WindowType *w, RectType *r, char *fname)
{
    SM_SaveScreenAsPCX(fname);
}


/*****************************************************************************
**
** WindowProc
**
** The image window's window procedure, called on just about every pass
** through the event loop.  All it does is update the cursor position label
*/
int WindowProc(WindowType *w, EventType *event)
{
    char    str[8];
    ImageDataType   *imgdata;       /* Image data attached to window    */
    static int  oldx = -1, oldy = -1, active = 0;

    imgdata = (ImageDataType *)SM_GetUserData(w);
    if (event->GraphX >=0
    && event->GraphY >=0
    && event->GraphX < GR_GetCanvasWidth(w)
    && event->GraphY < GR_GetCanvasDepth(w)) {
        if (event->GraphX != oldx || event->GraphY != oldy) {
            sprintf(str, "%d,%d",
                    (int)((double)event->GraphX/imgdata->zoom),
                    (int)((double)event->GraphY/imgdata->zoom));
            SM_SetLabelString(imgdata->cursorlabel, str);
            SM_RefreshGadgets(w);
            oldx = event->GraphX;
            oldy = event->GraphY;
            active = 1;
        }
    }
    else if (active) {
        imgdata = (ImageDataType *)SM_GetUserData(w);
        SM_EraseLabel(imgdata->cursorlabel);
        SM_RefreshGadgets(w);
        active = 0;
    }
    return event->Type;
}


