/*
** wintesto.c
**
** Test of event return values
**
**
** Copyright 1994 by
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
** All rights reserved
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <smtypes.h>
#include <smwindow.h>
#include <smevent.h>
#include <smlabel.h>
#include <smfont.h>

#ifdef __TURBOC__
extern unsigned _stklen = 16384U;
#endif

extern FILE *ferr;


int main(int argc, char **argv)
{
    WindowType  *w;
    LabelType   *label1, *label2, *label3, *label4, *label5, *label6;
    char        msgstr[80], typestr[80], managestr[80], regionstr[80];
    RectType    r;
    EventType   ev;
    int         msg;
    int         oldtype = -1, oldmsg = -1, oldman = -1, oldreg = -1;
    int         oldshift = -1, oldbuttonst = -1;
    int         oldx = -1, oldy = -1;

    ferr = NULL;
    SM_Init(0,0);

    SM_OpenApplication("TWS Demo : Event return values", NULL);

    r.Xmin = r.Ymin = 150;
    r.Xmax = r.Ymax = 400;

    w = SM_NewWindow(&r, "Event Demo", DOCUMENT | SCROLLATTRIB, NULL, NULL);

    managestr[0] = 0;
    r.Xmin = r.Ymin = 1;
    r.Xmax = 249;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label1 = SM_CreateLabel(w,
                            &r,
                            "Msg returned :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label2 = SM_CreateLabel(w,
                            &r,
                            "Event type :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label3 = SM_CreateLabel(w,
                            &r,
                            "Window cursor pos :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label4 = SM_CreateLabel(w,
                            &r,
                            "Region :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label5 = SM_CreateLabel(w,
                            &r,
                            "Shift State :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);
    r.Ymin = r.Ymax + 2;
    r.Ymax = r.Ymin + SM_GetSystemFontHeight() + 4;
    label6 = SM_CreateLabel(w,
                            &r,
                            "Button State :",
                            NULL,
                            SM_GetSystemFont(),
                            ALIGNLEFT,
                            False,
                            False,
                            False,
                            NULL);

    SM_SetWindowScrollbarMinMax(w, HSCROLLBAR, -100, 450);
    SM_SetWindowScrollbarMinMax(w, VSCROLLBAR, -100, 450);
    SM_OpenWindow(w);

    if (ferr == NULL) {
        ferr = fopen("wintesto.out","wb");
    }

    while (True) {
        SM_GetNextEvent(&ev);
        msg = SM_ProcessEvent(&ev);
        if (msg && (msg != oldmsg)) {
            sprintf(msgstr, "Msg returned : %d", msg);
            fprintf(ferr, "%s\n",msgstr);
            SM_SetLabelString(label1, msgstr);
            oldmsg = msg;
        }
        if (ev.Type != oldtype) {
            switch(ev.Type) {
                case BUTTONPRESS:
                    sprintf(typestr, "Event type : Button Press");
                    break;
                case BUTTONRELEASE:
                    sprintf(typestr, "Event type : Button Release");
                    break;
                case KEYPRESS:
                    sprintf(typestr, "Event type : Keypress");
                    break;
                case CLOCKEVENT :
                    sprintf(typestr, "Event type : Clock Tick");
                    break;
                case BUTTONMOTION :
                    sprintf(typestr, "Event type : Button Motion");
                    break;
                case DRAGMOTION :
                    sprintf(typestr, "Event type : Drag Motion");
                    break;
            }
            fprintf(ferr, "%s\n", typestr);
            SM_SetLabelString(label2, typestr);
            oldtype = ev.Type;
        }
        if (ev.Region != oldreg) {
            switch(ev.Region) {
                case UP :
                    sprintf(regionstr, "Region : UP");
                    break;
                case DOWN :
                    sprintf(regionstr, "Region : DOWN");
                    break;
                case LEFT :
                    sprintf(regionstr, "Region : LEFT");
                    break;
                case RIGHT :
                    sprintf(regionstr, "Region : RIGHT");
                    break;
                case UPLEFT :
                    sprintf(regionstr, "Region : UPLEFT");
                    break;
                case UPRIGHT :
                    sprintf(regionstr, "Region : UPRIGHT");
                    break;
                case DOWNLEFT :
                    sprintf(regionstr, "Region : DOWNLEFT");
                    break;
                case DOWNRIGHT :
                    sprintf(regionstr, "Region : DOWNRIGHT");
                    break;
                case MOVE_REGION :
                    sprintf(regionstr, "Region : MOVE_REGION");
                    break;
                case CLOSE_REGION :
                    sprintf(regionstr, "Region : CLOSE_REGION");
                    break;
                case MENU_REGION :
                    sprintf(regionstr, "Region : MENU_REGION");
                    break;
                case BASE_MENU_REGION :
                    sprintf(regionstr, "Region : BASE_MENU_REGION");
                    break;
                case WINDOW_REGION :
                    sprintf(regionstr, "Region : WINDOW_REGION");
                    break;
                case MAXIMIZE_REGION :
                    sprintf(regionstr, "Region : MAXIMIZE_REGION");
                    break;
                case CONTENT_REGION :
                    sprintf(regionstr, "Region : CONTENT_REGION");
                    break;
                case WORKSPACE_REGION :
                    sprintf(regionstr, "Region : WORKSPACE_REGION");
                    break;
                case MINIMIZE_REGION :
                    sprintf(regionstr, "Region : MINIMIZE_REGION");
                    break;
                case CANVAS_REGION :
                    sprintf(regionstr, "Region : CANVAS_REGION");
                    break;
                case APP_TITLE_REGION :
                    sprintf(regionstr, "Region : APP_TITLE_REGION");
                    break;
            }
            SM_SetLabelString(label4, regionstr);
            oldreg = ev.Region;
            fprintf(ferr, "%s\n", regionstr);
        }
        if (ev.ShiftState != oldshift) {
            sprintf(regionstr, "Shift State : %x", ev.ShiftState);
            SM_SetLabelString(label5, regionstr);
            oldshift = ev.ShiftState;
        }
        if (ev.ButtonState != oldbuttonst) {
            switch (ev.ButtonState) {
                case LEFTBUTTONACTIVE :
                    sprintf(regionstr, "Left button active");
                    break;
                case RIGHTBUTTONACTIVE :
                    sprintf(regionstr, "Right button active");
                    break;
                case MIDDLEBUTTONACTIVE :
                    sprintf(regionstr, "Middle button active");
                    break;
                default :
                    sprintf(regionstr, "Multiple or no buttons active");
                    break;
            }
            SM_SetLabelString(label6, regionstr);
            oldbuttonst = ev.ButtonState;
        }
        if (ev.WinX != oldx || ev.WinY != oldy) {
            sprintf(regionstr, "Window cursor pos : %d,%d",ev.WinX, ev.WinY);
            SM_SetLabelString(label3, regionstr);
            oldx = ev.WinX;
            oldy = ev.WinY;
        }
                            

        if (msg == KEYPRESS) {
            if (ev.ASCII == 'q') {
                SM_Exit(NULL);
            }
        }
    }
}




