/*
** wintestq.c
**
** Test of the Hotregion gadget
**
** Copyright 1995
** Dean Clark
** PO Box 37464
** Albuquerque, NM  87176
*/

#include    <stdio.h>
#include    <stdlib.h>
#include    <mem.h>

#include    <smwindow.h>
#include    <smevent.h>
#include    <hotregn.h>
#include    <smlabel.h>
#include    <smfont.h>
#include    <smgraph.h>
#include    <smgadget.h>

int ShowPosition(HotregionType *region);
int ShowSelected(HotregionType *region);
int DrawRegion(HotregionType *region);

typedef struct {
    LabelType   *lblPosition;
    LabelType   *lblSelected;
} RegionData;

/*
** Borlandc requires extra stack
*/
#ifdef __TURBOC__
extern unsigned _stklen = 14336U;
#endif

void PrntScrn(WindowType *w, RectType *r, char *fname);;


int main (int argc, char *argv[])
{
    HotregionType   *region;
    int             points[10];
    EventType       event;
    int             msg;
    RectType        labelrect;
    WindowType      *w;
    RegionData      data;
    
    /*
    ** Initialize TWS and the graphics hardware; use the .CFG parameters
    */
    SM_Init(0,0);
    
    /*
    ** Open the application
    */
    SM_OpenApplication("TWS : Hotregion Example", NULL);
    
    /*
    ** Get the workspace window
    */
    w = SM_WorkspaceWindow();

    labelrect.Xmin = 100;
    labelrect.Ymin = 85;
    labelrect.Xmax = 250;
    labelrect.Ymax = 210;
    GR_CreateGraphState(w, &labelrect, SMDEFAULT);
    
    /*
    ** Create a label to display output
    */
    labelrect.Xmin = labelrect.Ymin = 5;
    labelrect.Xmax = 225;
    labelrect.Ymax = 35;
    data.lblPosition = SM_CreateLabel(w,
                                      &labelrect,
                                      "",
                                      NULL,
                                      SM_GetSystemFont(),
                                      ALIGNCENTER,
                                      False,
                                      False,
                                      BEVELED,
                                      NULL);
    labelrect.Ymin = labelrect.Ymax + 5;
    labelrect.Ymax = labelrect.Ymin + 30;
    data.lblSelected = SM_CreateLabel(w,
                                      &labelrect,
                                      "",
                                      NULL,
                                      SM_GetSystemFont(),
                                      ALIGNCENTER,
                                      False,
                                      False,
                                      BEVELED,
                                      NULL);
                           
    /*
    ** Create a hotregion; first set the point list
    ** Make sure the polygon is closed
    */
    points[0] = 100;
    points[1] = 100;
    points[2] = 160;
    points[3] = 85;
    points[4] = 240;
    points[5] = 130;
    points[6] = 180;
    points[7] = 200;
    points[8] = 100;
    points[9] = 100;
    region = SM_CreateHotregion(w,
                                5,
                                points,
                                (void *)&data,
                                DrawRegion,         /* Draw                 */
                                ShowPosition,       /* Enter                */
                                ShowPosition,       /* Leave                */
                                NULL,               /* Drag                 */
                                ShowSelected);      /* Select               */

    SM_SetPrintscreenProc(NULL, NULL, "hotregn.pcx", PrntScrn);
    /*
    ** Event loop
    */    
    while (1) {
        SM_GetNextEvent(&event);
        msg = SM_ProcessEvent(&event);
        if (msg & KEYPRESS) {
            if (event.ASCII == 'q') {
                SM_Exit(NULL);
            }
        }
    }        
}


/*****************************************************************************
**
** ShowPosition
**
** This function is called on Hotregion ENTER and LEAVE events.
*/
int ShowPosition(HotregionType *region)
{
    RegionData      *data;
    
    data = (RegionData *)SM_GetHotregionData(region);
    if (region->inside) {
        SM_SetLabelString(data->lblPosition, "ENTERING Region");
    }
    else {
        SM_SetLabelString(data->lblPosition, "LEAVING Region");
    }
    return 1;
}



/*****************************************************************************
**
** ShowSelected
**
** Function called when user clicks in the Hotregion
*/
int ShowSelected(HotregionType *region)
{
    RegionData  *data;
    static int  count = 0;
    char        str[35];

    data = (RegionData *)SM_GetHotregionData(region);
    count++;
    sprintf(str, "Region selected %d times", count);
    SM_SetLabelString(data->lblSelected, str);
    return 1;
}


/*****************************************************************************
**
** DrawRegion
**
** Called when the region needs to be redrawn
*/
int DrawRegion(HotregionType *region)
{
    int     *pts;
    int     i, n;
    PointType   *regionpts;
    WindowType  *w;

    n = SM_GetHotregionNPts(region);
    pts = (int *)malloc(2*n*sizeof(int));
    w = SM_GetGadgetWindow(region);
    regionpts = SM_GetHotregionPts(region);
    for (i = 0; i < n; i++) {
        SM_PtContentToCanvas(w,
                             regionpts[i].X, regionpts[i].Y,
                             &(pts[i*2]), &(pts[i*2+1]));
    }
    GR_DrawFilledPolygon(SM_GetGadgetWindow(region),
                         SM_GetHotregionNPts(region),
                         pts);
    free(pts);
    return 0;
}

void PrntScrn(WindowType *w, RectType *r, char *fname)
{
    SM_SaveScreenAsPCX(fname);
}


