/*
** Example of zooming a window
**
*/

#include    <stdlib.h>
#include    <stdio.h>
#include    <smwindow.h>
#include    <smevent.h>
#include    <smtypes.h>
#include    <smlabel.h>
#include    <smeditst.h>
#include    <smfont.h>
#include    <smgadget.h>

int main(int argc, char *argv[])
{
    RectType    winrect;
    WindowType  *w;
    GadgetType  *g;
    EventType   *ev;
    int         msg;
    int         newx, newy, newwidth, newheight;
    int         fontw, fonth;

    /*
    ** Initialize graphics system, workspace.  All windows and gadgets
    ** will use the system default font.  The configuration file 'tws.cfg'
    ** should have a stroked font specified as the system font.
    */
    SM_Init(argc, argv);
    SM_OpenApplication("'Zoom' Window demo", NULL);

    /*
    ** Build a window
    */
    winrect.Xmin = 100;                 /* Set up a window                  */
    winrect.Xmax = winrect.Xmin + 450;
    winrect.Ymin = 150;
    winrect.Ymax = winrect.Ymin + 250;
    w = SM_NewWindow(&winrect, "Zoom Window", DOCUMENT, NULL, NULL);

    /*
    ** Add a couple of gadgets.  Notice that in this case it's not necessary to
    ** save the returned gadget pointers
    */
    winrect.Xmin = winrect.Ymin = 2;
    winrect.Xmax = winrect.Xmin + 150;
    winrect.Ymax = winrect.Ymin + 25;
    SM_CreateLabel(w,
                   &winrect,
                   "Label 1",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);
    winrect.Ymin = winrect.Ymax + 5;
    winrect.Ymax = winrect.Ymin + 25;
    SM_CreateLabel(w,
                   &winrect,
                   "Label 2",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);
    winrect.Xmin = winrect.Xmax + 5;
    winrect.Xmax = winrect.Xmin + 150;
    SM_CreateLabel(w,
                   &winrect,
                   "Label 3",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);
    winrect.Ymin = 2;
    winrect.Ymax = winrect.Ymin + 25;
    SM_CreateLabel(w,
                   &winrect,
                   "Label 3",
                   NULL,
                   SM_GetSystemFont(),
                   ALIGNCENTER,
                   False,
                   False,
                   True,
                   NULL);

    /*
    ** Display the window
    */
    SM_OpenWindow(w);

    /*
    ** The event loop.  Every time a keyboard key is pressed, zoom the
    ** window a little more.  If the 'q' key is pressed then quit.
    */
    while (1) {
        ev = SM_GetNextEvent();
        msg = SM_ProcessEvent(ev);
        if (msg == KEYPRESS) {
            if (ev->ASCII == 'q') {
                SM_Exit(NULL);
            }
            /*
            ** Otherwise, loop through the window gadgets and change
            ** their size and location, and change the font size.
            ** NOTE: The first gadget is a dummy...
            */
            g = SM_GetGadgets(w);
            g = g->next;
            while (g) {
                newx = g->bound.Xmin * 0.25;
                newy = g->bound.Ymin * 0.25;
                if (newx == 0) {
                    newx++;
                }
                if (newy == 0) {
                    newy++;
                }
                newwidth = (g->bound.Xmax - g->bound.Xmin) * 0.25;
                newheight = (g->bound.Ymax - g->bound.Ymin) * 0.25;
                SM_AdjustGadgetBound(g->gadget, newx, newy, newwidth, newheight);
                SM_GetGadgetFontSize(g->gadget, &fontw, &fonth);
                SM_SetGadgetFontSize(g->gadget, fontw*1.25, fonth*1.25);
                g = g->next;
            }
            /*
            ** Force the window to redraw itself
            */
            SM_RedrawWindow(w);
        }
    }
}




